/***********************************************************************
* Name:
*        dapint64.c
*
* Function:
*        Operators on Large (64-bit) Integers
*
* Public Routines:
*        Incr64()
*        Decr64()
*        Add64()
*        Sub64()
*        ShiftR64()
*        ShiftL64()
*        Mul64()
*        Div64()
*        I64toCh()
*
***********************************************************************/

/* Standard (ANSI) C library header files. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Application header files. */
#include "dsmapitd.h"
#include "dsmapifp.h"
typedef uint64 int64;
#include "dapint64.h"

/***********************************************************************
*        Public Routines
***********************************************************************/

/*----------------------------------------------------------------------
* Name:    Make64()
*
* Action:  Construct a 64-bit integer from a 32-bit integer.
*
* Input:   n - 32 bit integer to be used to construct 64-bit integer.
*
* Return
* Value:   Constructed 64-bit integer.
*
* Side
* Effects: None.
*
* Notes:   All int64's are assumed to be unsigned.
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int64
Make64( n )
   uint32 n;
#else
int64
Make64( uint32 n )
#endif

{
  int64 result;

  result.hi = 0;
  result.lo = n;

  return ( result );
} /* Make64() */

/*----------------------------------------------------------------------
* Name:    Incr64()
*
* Action:  Increment a 64-bit integer by some 32-bit integer.
*
* Input:   x - 64-bit integer to be incremented.
*          n - 32-bit increment.
*
* Return
* Value:   Result of increment operation.
*
* Side
* Effects: None.
*
* Notes:   All int64's are assumed to be unsigned.
*          Assume no overflow of sum.
*          Rewritten for efficiency
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int64
Incr64( x, n )
   int64  x;
   uint32 n;
#else
int64
Incr64( int64 x, uint32 n )
#endif
{
  x.lo += n;                    /* add lo order word */

  if ( x.lo < n )               /* did the add cause an overflow? */
    x.hi++;                     /* yes, adjust hi word for carry */

  return ( x );
} /* Incr64() */

/*----------------------------------------------------------------------
* Name:    Decr64()
*
* Action:  Decrement a 64-bit integer by some 32-bit integer.
*
* Input:   x - 64-bit integer to be decremented.
*          n - 32-bit decrement.
*
* Return
* Value:   Result of decrement operation.
*
* Side
* Effects: largeIntP - replaced by the result of the decrement
*                      operation.
*
* Notes:   All int64's are assumed to be unsigned.
*          Assume no underflow of difference.
*          Rewritten for efficiency
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int64
Decr64( x, n )
   int64  x;
   uint32 n;
#else
int64
Decr64( int64 x, uint32 n )
#endif
{

  if ( x.lo < n )               /* is a borrow required? */
    x.hi--;                     /* yes, borrow from hi word */

  x.lo -= n;                    /* subtract lo order word */

  return ( x );
} /* Decr64() */

/*----------------------------------------------------------------------
* Name:    Add64()
*
* Action:  Add two 64-bit integers.
*
* Input:   x - first 64-bit integer.
*          y - second 64-bit integer.
*
* Return
* Value:   Result of addition operation.
*
* Side
* Effects: None.
*
* Notes:   All int64's are assumed to be unsigned.
*          Assume no overflow of sum.
*          Rewritten for efficiency
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int64
Add64( x, y )
   int64 x;
   int64 y;
#else
int64
Add64( int64 x, int64 y )
#endif
{
  x = Incr64( x, y.lo);
  x.hi += y.hi;

  return ( x );
} /* Add64() */

/*----------------------------------------------------------------------
* Name:    Sub64()
*
* Action:  Subtract two 64-bit integers.
*
* Input:   x - first 64-bit integer.
*          y - second 64-bit integer.
*
* Return
* Value:   Result of subtracting second 64-bit integer from the first.
*
* Side
* Effects: None.
*
* Notes:   All int64's are assumed to be unsigned.
*          Assume no underflow of difference (ie. x >= y)
*          Rewritten for efficiency
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int64
Sub64( x, y )
   int64 x;
   int64 y;
#else
int64
Sub64( int64 x, int64 y )
#endif
{
  x = Decr64( x, y.lo);
  x.hi -= y.hi;

  return ( x );
} /* Sub64() */

/*----------------------------------------------------------------------
* Name:    ShiftR64()
*
* Action:  Performs a logical shift right on an int64 by b bits.
*
* Input:   x - the address of the 64-bit integer.
*          b - number bits to shift.
*
* Return
* Value:   the value passed in is shifted
*
* Side
* Effects: None.
*
* Notes:   All int64's are assumed to be unsigned.
*        As expected, bits shifted below bit 0 go into the bit bucket.
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void ShiftR64(x, b)
   int64         *x;
   unsigned int  b;
#else
void ShiftR64(int64 *x, unsigned int b)
#endif
{
  if (b >= 32)
  {
    x->lo = x->hi;
    x->hi = 0;
    b &= 0x1F;                 /* get rid of all bits above value 32  */
  }
  x->lo >>= b;                 /* shift the low word into position    */
  x->lo |= (x->hi << (32-b));  /* OR in the bits remaining of hi word */
  x->hi >>= b;                 /* shift the hi word by x bits         */
}  /* end ShiftR64() */

/*----------------------------------------------------------------------
* Name:    ShiftL64()
*
* Action:  Performs a logical shift left on an int64 by b bits.
*
* Input:   x - the address of the 64-bit integer.
*          b - number of bits to shift.
*
* Return
* Value:   The value passed in is shifted
*
* Side
* Effects: None.
*
* Notes:   All int64's are assumed to be unsigned.
*        As expected, bits shifted above bit 63 go into the bit bucket.
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void ShiftL64(x, b)
   int64         *x;
   unsigned int  b;
#else
void ShiftL64(int64 *x, unsigned int b)
#endif
{
  if (b >= 32)
  {
    x->hi = x->lo;
    x->lo = 0;
    b &= 0x1F;                 /* get rid of all bits above value 32  */
  }
  x->hi <<= b;                 /* shift the hi word into position     */
  x->hi |= (x->lo >> (32-b));  /* OR in the bits remaining of lo word */
  x->lo <<= b;                 /* shift the lo word by x bits         */
}  /* end ShiftL64() */

/*----------------------------------------------------------------------
* Name:    Mul64()
*
* Action:  Multiply two 64 bit integers.
*
* Input:   x - first 64 bit integer
*          y - second 64 bit integer
*          overflow - address of an integer
*
* Return
* Values:  1) 64 bit Product (x * y) is returned
*          2) overflow set to 0 or 1 to indicate if Product exceeded
*             64 bits (1 indicates overflow)
*
* Side
* Effects: None.
*
* Notes:   All int64's are assumed to be unsigned.
*          Any overflow above 64 bits will be lost.
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int64
Mul64(x, y, overflow)
   int64 x;
   int64 y;
   int   *overflow;
#else
int64
Mul64(int64 x, int64 y, int *overflow)
#endif
{
int64  Prod;
int    bits_lost = 0;            /* 1 = bits have been lost on hi end */

  *overflow = 0;                        /* start w/ no overflow */
  Prod.hi = 0;                          /* initialize Product  */
  Prod.lo = 0;

  do {
     if (y.lo & 0x1)  /* if bit 0 is 1, curr power of 2 will be added */
     {
        /* now that we're adding again, if there have been any bits   */
        /* lost from hi end of x, we know there's an overflow.        */
        if (bits_lost)
           *overflow = 1;
        Prod = Add64(Prod,x);         /* add shifted x to the product */
     }
     if (x.hi & 0x80000000)    /* if bit 63 of x is on before ShiftL, */
        bits_lost = 1;         /* indicate bits have been lost */
     ShiftL64(&x,1);           /* double the value being multiplied   */
     ShiftR64(&y,1);           /* shift next test bit into position   */
  } while (y.hi || y.lo);    /* until all y bits are 0 (ie.Mult done) */

  return ( Prod );
}  /* end Mul64() */

/*----------------------------------------------------------------------
* Name:    Div64()
*
* Action:  Divide two 64 bit integers.
*
* Input:   x - first 64 bit integer
*          y - second 64 bit integer
*
* Return
* Values:  1) Quotient is returned
*          2) Remainder is stored in specified address
*
* Side
* Effects: None.
*
* Notes:   All int64's are assumed to be unsigned.
*          As usual, bits shifted below bit 0 go into the bit bucket.
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int64
Div64(x, y, rmdrP)
   int64 x;
   int64 y;
   int64 *rmdrP;
#else
int64
Div64(int64 x, int64 y, int64 *rmdrP)
#endif
{
int64  Quot,    /* the final answer is accumulated here       */
       tQuot,   /* temp. quotient to find a single power of 2 */
                /*  that fits into the dividend on each pass  */
       divisor;

  Quot.hi = 0;                          /* initialize Quotient */
  Quot.lo = 0;

  if (y.hi==0 && y.lo==0)               /* check for divide by zero */
  {
     y.hi = 1/y.lo;         /* force a divide by zero so that error-  */
                            /*  handling will be platform-specific    */
  }
  else  /* do the division */
  {
     while (Cmp64(x,y) != LESSTHAN)        /* while x >= y            */

     {
    /* find largest Y (multiplied by powers of 2) that is less than X */
        divisor = y;                    /* copy Y so it is unchanged  */
        tQuot.hi = 0; /* init temp quotient (at least 1y goes into x) */
        tQuot.lo = 1;
        while (Cmp64(x,divisor) != LESSTHAN  &&
                        (divisor.hi <= 0x7FFFFFFF))
        {
           ShiftL64(&divisor,1);      /* double divisor (2y 4y 8y...) */
           ShiftL64(&tQuot,1);        /* count number of y's so far   */
        }
        ShiftR64(&divisor,1); /* back up to a value that's less than X*/
        ShiftR64(&tQuot,1);

        Quot = Add64(Quot,tQuot);      /* calc cumulative quotient    */
        x = Sub64(x,divisor);          /* deduct the multiples so far */
     }
     *rmdrP = x;               /* save remainder in specified address */
  }
  return ( Quot );
}  /* end Div64() */

/*----------------------------------------------------------------------
* Name:    I64toCh()
*
* Action:  Converts an int64 to it's character respresentation.
*          Currently only decimal is supported.
*
* Input: value - the 64 bit integer
*        string - a pointer to a character output buffer
*        radix - the base to be used in converting value.  Hexadecimal
*                 can be easily formatted using printf("%08lX", ... )
*
* Return
* Values:  returns the address of the null-terminated string  or
*          NULL if an error occurred
*
* Side
* Effects: None.
*
* Notes: - the space for string must be long enough to hold the
*          converted value AND the null terminator.  For decimal, this
*          may be up to 21 characters.  For octal, it may be up to 23.
*        - tempstr is an internal buffer of maximum length to build the
*          decimal string so that if the caller *knows* they are
*          converting a number that will require less than the maximum,
*          such as 365, they only need space for four characters.
*        - All int64's are assumed to be unsigned.
*        - This routine was originally written to convert to both
*          decimal and octal, but due to it's rarity, octal support has
*          been commented out.  I have left the code for octal, but
*          it is commented out.
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
char *I64toCh(value, string, radix)
   int64 value;
   char  *string;
   int   radix;
#else
char *I64toCh(int64 value, char *string, int radix)
#endif
{
char *cptr;
char tempstr[24];  /* max. of 23 characters plus one for good measure */

  if (radix != 10)
     return NULL;

  cptr = tempstr + sizeof(tempstr)-1;       /* point to end of string */
  *cptr = '\0';                  /* initialize null-terminated string */

  /*--- DECIMAL conversion -------------------------------------------*/
  {
  int64   rem, base;       /* these vars are only used for base 10 */
    base.hi = 0;             /* create an int64 divisor */
    base.lo = 10;
    do                  /* repeatedly divide by 10 until nothing left.*/
    {                   /* after each divide, the rmdr is the digit   */
       value = Div64(value, base, &rem);
       cptr--;
       *cptr = (char)(rem.lo + '0');
    } while (value.lo || value.hi);
  }
  /*------------------------------------------------------------------*/

  /*--- OCTAL conversion ---------------------------------------------*/
  /** do
   ** {
   **    cptr--;
   **    *cptr = (char)((value.lo & 0x07) + '0');
   **    ShiftR64(&value,3);
   ** } while (value.lo || value.hi);         */
  /*------------------------------------------------------------------*/

  strcpy(string,cptr);      /* copy new string into caller's string */
  return (string);
}  /* end I64toCh() */

/*----------------------------------------------------------------------
* Name:    ChtoI64()
*
* Action:  Converts a string of character digits into an int64.
*          Currently only decimal is supported.
*
* Input:   string - a pointer to a character string to be converted.
*          radix  - radix to be used (2 - 10).
*
* Return
* Values:  The equivalent int64 value, or zero if a non-digit character
*          was encountered.
*
* Side
* Effects: None.
*
* Notes: - All int64's are assumed to be unsigned.
*---------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int64
ChtoI64( str, radix )
   char *str;
   int  radix;
#else
int64
ChtoI64( char *str, int radix )
#endif

{
  int64 power, radix64, result;
  int   digit, ovfl, len;

  power   = Make64( 1 );
  radix64 = Make64( radix );
  result  = Make64( 0 );
  len     = strlen( str );

  while ( --len >= 0 )
  {
    digit = ((int) str[len]) - '0';

    if ( digit < 0 || digit >= radix )
      return ( Make64( 0 ) );

    result = Add64( result, Mul64( power, Make64( digit ), &ovfl ) );
    power  = Mul64( power, radix64, &ovfl );
  }

  return ( result );
} /* ChtoI64() */
