/***********************************************************************
* Name:
*        dapiqry.c
*
* Function:
*        Functions invoked via menu selections
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        The file contains the functions that are invoked from the
*        menu processing code and tables.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "dsmapitd.h"          /* Standard DSM API header.      */
#include "dsmapifp.h"
#include "dapitype.h"        /* Sample API data types.        */
#include "dsmrc.h"           /* DSM API return codes.         */
typedef uint64 int64;
#include "dapint64.h"        /* 64 bit integer functions.     */
#include "dapiutil.h"        /* Various utility functions.    */
/*----------------------------------------------------------------------+
| Private typedefs
.----------------------------------------------------------------------*/
typedef struct
{
   char *fs;
   char *hl;
   char *ll;
   uint8 objType;
} dsmObjNameP;

/*----------------------------------------------------------------------+
| Linked list structure to store restore/retrieve queries
.----------------------------------------------------------------------*/
typedef struct SGetInfo
{
   struct SGetInfo   *next;           /* Next item in list, null if last  */
   uint64            item64;          /* Number of item in list.          */
   dsmObjNameP       *objName;        /* Name of object                   */
   ObjID             objId;           /* Id of object for restore.        */
   uint32            copyGroup;       /* Needed to delete object          */
   uint64            restoreOrder;    /* Sort field for optimum restore.  */
   uint8             mediaClass;      /* Item offline/online/other?       */
   uint16            objInfolen;      /* Length of buffer below.          */
   char              *objInfo;        /* Not a null terminated string!    */
} GetInfo;

/*----------------------------------------------------------------------+
| Linked list structure to store fs queries.  Not in 64bit format!
.----------------------------------------------------------------------*/
typedef struct SfsInfo
{
   struct SfsInfo    *next;           /* Next item in list, null if last  */
   uint32            item;            /* Number of item in list.          */
   char              *fsName;         /* Name of fs                       */
} fsInfo;

/*----------------------------------------------------------------------+
| Structure to maintain data on current state of validation check
.----------------------------------------------------------------------*/
typedef struct
{
   char    *seed;            /* Pointer to the seed string.          */
   uint16  seed_len;         /* Length of seed string.               */
   char    *seed_cursor;     /* Position of validate in seed string. */
   uint64  total_sz64;       /* Total size thus far of verified obj. */
   uint64  orig_sz;          /* Orig size of obj as sent by api.     */
   uint32  seed_string_size ;/* Size of entire seed string           */
   bool_t  first_time;       /* First time checking for partial obj? */
} seedValidate;

#ifdef _NO_PROTO_
extern void rcApiOut();
#else
extern void rcApiOut(int16 rc);
#endif
/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void  QryDateToStr();
static int16 QryAddGetLlist();
static int16 QryFreeGetLlist();
static int16 QryFreeGetLlistItem();
static int16 QryDoGet();
static int16 QryDoDelete();
static int16 QryDoRestoreObjs();
static int16 Qry_Validate_Data();
static int16 QryBeginTxn();
static void  QryPrintMediaClass();
static void  QryPrintObjType();
static void  QryPrintObjInfo();
static void  QryPrintObjState();
static void  QryPrintCopySer();
static void  QryPrintCopyMode();
static int16 QryAddFsLlist();
static int16 QryFreeFsLlist();
static int16 QryDoFsDelete();
static int16 QryMakeSeed();
#else
static void QryDateToStr(char *s,dsmDate date);

static int16 QryAddGetLlist(uint16 get_type,
                            uint64 qry_item64,
                            GetInfo **llHead,
                            GetInfo **llTail,
                            char *buff);

static int16 QryFreeGetLlist(GetInfo **llHead,
                             GetInfo **llTail);

static int16 QryFreeGetLlistItem(GetInfo **item);

static int16 QryDoGet(uint16 get_type,
                      GetInfo *llHead,
                      GetInfo *llTail,
                      uint64  items64,
                      bool_t  mountWait);

static int16 QryDoDelete(uint16 delete_type,
                         GetInfo *llHead,
                         GetInfo *llTail,
                         uint64  items64);

static  int16 QryDoRestoreObjs(uint16 get_type,
                               GetInfo *llStart,
                               char *rest_ibuff,
                               uint32 items,
                               bool_t mountWait,
                               char *part_pbuff);

static int16 Qry_Validate_Data(DataBlk *dataBlk,
                               seedValidate *seedV,
                               uint32 item,
                               char *part_pbuff);
static int16 QryBeginTxn(uint32  dsmHandle,        /* DSM API sess handle  */
                         uint32  *txn_objs);       /* Objects in this txn. */
static void QryPrintMediaClass(uint8 mediaClass);
static void QryPrintObjType(uint8 objType);
static void  QryPrintObjInfo(uint16 objInfolen, char *objInfo);
static void QryPrintObjState(uint8 objState);
static void QryPrintCopySer(uint8 copySer);
static void QryPrintCopyMode(uint8 copyMode);
static int16 QryAddFsLlist(uint32  qry_item32,
                           fsInfo **llHead,
                           fsInfo **llTail,
                           qryRespFSData *fsResp);
static int16 QryFreeFsLlist(fsInfo **llHead,
                            fsInfo **llTail);
static int16 QryDoFsDelete(fsInfo *llHead,
                           fsInfo *llTail,
                           uint32  items32);
static int16 QryMakeSeed(seedValidate  *seedV,
                         char *seed_str );
#endif

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    perform_query_fs()
|
| Action:  Take the information passed in the dialog to this function,
|          build the backup objects dynamically, and send them to the
|          server.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A dsm api call failed stopping execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 perform_query_fs(sel_dialog, fs_action)
   dialog  *sel_dialog;
   uint16  fs_action;
#else
int16 perform_query_fs(dialog *sel_dialog,uint16 fs_action)
#endif
{
dialog     *dlg;
uint32      i;
int16       rc;
qryFSData   qFSData;
DataBlk     qData;
qryRespFSData qRespFSData, *fsResp;
char     *fs;
bool_t   first_fs = bTrue;
fsInfo   *fsllHead = NULL;
fsInfo   *fsllTail = NULL;
fsInfo   *fsll     = NULL;
uint32        qry_item = 0;

char fs_head[100];
char fs_div[100];
char fs_info[DSM_MAX_FSINFO_LENGTH+1]; /* copy array into here to make string*/

strcpy(fs_head, "Filespace Name                 Type       Occupancy  Capacity   Info");
strcpy(fs_div,  "------------------------------ ---------- ---------- ---------- -----------");

if (!dsmHandle)
{
   printf("You must signon before performing this action.\n");
   return(0);
}

if (!(fs = malloc(DSM_MAX_FSNAME_LENGTH + 1))) return (RC_NO_MEMORY);
memset(fs,0x00,(DSM_MAX_FSNAME_LENGTH + 1));

/* First pull all values out of the passed dialog for our use.  */
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != DSMAPI_END)
{
   switch (dlg->item_type)
   {
      case DSMAPI_FILESPACE :
        if ((strlen(dlg->item_buff) > DSM_MAX_FSNAME_LENGTH)
           || !*dlg->item_buff)
         {
            strcpy(fs,"*");
         }
         else strcpy(fs,dlg->item_buff);
         break;
      default :
         ;
   }
   i++;
   dlg = &sel_dialog[i];
}

qFSData.stVersion = qryFSDataVersion;
qFSData.fsName    = fs;

qData.stVersion = DataBlkVersion;
qData.bufferLen = sizeof(qryRespFSData);
qData.bufferPtr = (char *)&qRespFSData;

qRespFSData.stVersion = qryRespFSDataVersion;

if ((rc = dsmBeginQuery(dsmHandle,qtFilespace,(dsmQueryBuff *)&qFSData)))
{
   printf("*** dsmBeginQuery failed: \n");
   rcApiOut(rc);
   rc = (RC_SESSION_FAILED);
}
else
{
   while ((rc = dsmGetNextQObj(dsmHandle,&qData)) == DSM_RC_MORE_DATA)
   {
        qry_item++;
        fsResp = (qryRespFSData *)qData.bufferPtr;
        switch(fs_action)
        {
           /*------------------------------------
           |  Display fs info from the query
           .-------------------------------------*/
           case (fs_query) :
              if (first_fs)
              {
                 printf("%s\n%s\n",fs_head,fs_div);
                 first_fs = bFalse;
              }

              FSInfoResp(fs_info, fsResp); /* format area into string */

              printf("%-30s %-10s %10lu-%10lu %s\n",
                  fsResp->fsName,
                  fsResp->fsType,
                  fsResp->occupancy.lo,
                  fsResp->capacity.lo,
                  fs_info);
              break;
           /*-----------------------------------------------------
           |  Save info from the query so we can delete the fs(s)
           .-------------------------------------------------------*/
           case (fs_delete) :
               rc = QryAddFsLlist(qry_item,
                                  &fsllHead,
                                  &fsllTail,
                                  fsResp);
               if (rc)
               {
               QryFreeFsLlist(&fsllHead,
                              &fsllTail);
               printf("FS Delete failed during the filespace query.\n");
               }
              break;
           /*-----------------------------------------------------
           |  Default processing.
           .-------------------------------------------------------*/
           default : ;
        }
   }
   if (rc && (rc != DSM_RC_FINISHED))
   {
      printf("\n*** dsmGetNextQObj failed: \n");
      rcApiOut(rc);
   }
   rc = dsmEndQuery(dsmHandle);
}

if (fs_delete)
{
   rc = QryDoFsDelete(fsllHead,fsllTail,qry_item);
   QryFreeFsLlist(&fsllHead,&fsllTail);
}
return rc;
}

/*----------------------------------------------------------------------+
| Name:    perform_query_mc()
|
| Action:  Take the information passed in the dialog to this function,
|          build the backup objects dynamically, and send them to the
|          server.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A dsm api call failed stopping execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 perform_query_mc(sel_dialog, parm1)
   dialog *sel_dialog;
   uint16 parm1;
#else
int16 perform_query_mc(dialog *sel_dialog,uint16 parm1)
#endif

{
dialog    *dlg;
uint32     i;
int16      rc;

qryMCData     qMCData;
DataBlk       qData;
qryRespMCDetailData qRespMCData, *mcResp;
char *mc, *s;
bool_t  done = bFalse;
uint32  qry_item;

char mc_head[50];
char mc_div[50];

strcpy(mc_head, "Mgmt Class Name                Description");
strcpy(mc_div,  "------------------------------ ------------------");

if (!dsmHandle)
  {
  printf("You must signon before performing this action.\n");
  return(0);
  }

if (!(mc = malloc(DSM_MAX_MC_NAME_LENGTH + 1))) return (RC_NO_MEMORY);
if (!(s = malloc(1024))) return (RC_NO_MEMORY);  /* Work buffer */

/* First pull all values out of the passed dialog for our use.  */
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != DSMAPI_END)
  {
    switch (dlg->item_type)
      {
      case DSMAPI_MC :
        if ((strlen(dlg->item_buff) > DSM_MAX_MC_NAME_LENGTH)
           || !*dlg->item_buff)
          {
          *mc = 0x00;       /* Default to all if null entered. */
          }
        else strcpy(mc,dlg->item_buff);
        break;
      default :
         ;
      }
    i++;
    dlg = &sel_dialog[i];
  }

/*--------------------------------------------------------------------
|  Fill in the qryMCData structure with the query criteria we want.
.---------------------------------------------------------------------*/
qMCData.stVersion = qryMCDataVersion;
qMCData.mcName    = mc;
qMCData.mcDetail  = bTrue;

qData.stVersion = DataBlkVersion;
qData.bufferLen = sizeof(qryRespMCDetailData);
qData.bufferPtr = (char *)&qRespMCData;

qRespMCData.stVersion = qryRespMCDetailDataVersion;

if ((rc = dsmBeginQuery(dsmHandle,qtMC,(dsmQueryBuff *)&qMCData)))
{
   printf("*** dsmBeginQuery failed: \n");
   rcApiOut(rc);
   rc = (RC_SESSION_FAILED);
}

else
{
   done = bFalse;
   qry_item = 0;
   while (!done)
   {
      rc = dsmGetNextQObj(dsmHandle,&qData);
      if ((   (rc == DSM_RC_OK)
           || (rc == DSM_RC_MORE_DATA)
           || (rc == DSM_RC_FINISHED))
          && qData.numBytes)
      {
         qry_item++;
         mcResp = (qryRespMCDetailData *)qData.bufferPtr;
         printf("Mgmt. Class %lu:\n",qry_item);
         printf("              Name: %s\n",mcResp->mcName);
         printf("       Description: %s\n",mcResp->mcDesc);

         printf("    Backup CG Name: %s\n",mcResp->backupDet.cgName);
         printf("            Frequency: %u\n",mcResp->backupDet.frequency);
         printf("     Ver. Data Exists: %u\n",mcResp->backupDet.verDataExst);
         printf("    Ver. Data Deleted: %u\n",mcResp->backupDet.verDataDltd);
         printf("     Retain Extra Ver: %u\n",mcResp->backupDet.retXtraVers);
         printf("      Retain Only Ver: %u\n",mcResp->backupDet.retOnlyVers);
         QryPrintCopySer(mcResp->backupDet.copySer);
         QryPrintCopyMode(mcResp->backupDet.copyMode);
         printf("     Copy Destination: %s\n",mcResp->backupDet.destName);

         printf("   Archive CG Name: %s\n",mcResp->archDet.cgName);
         printf("            Frequency: %u\n",mcResp->archDet.frequency);
         printf("      Retain versions: %u\n",mcResp->archDet.retainVers);
         QryPrintCopySer(mcResp->archDet.copySer);
         QryPrintCopyMode(mcResp->archDet.copyMode);
         printf("     Copy Destination: %s\n",mcResp->archDet.destName);
      }
      else
      {
         done = bTrue;
         if (rc != DSM_RC_FINISHED)
         {
            printf("*** dsmGetNextQObj failed: \n");
            rcApiOut(rc);
         }
      }
      if (rc == DSM_RC_FINISHED) done = bTrue;
   }
   rc = dsmEndQuery(dsmHandle);
}

return rc;

}

/*----------------------------------------------------------------------+
| Name:    perform_query_get()
|
| Action:  Take the information passed in the dialog to this function,
|          build the proper query, issue it, and display results to the
|          screen.  This handles 3 query types:  Backup, fast Backup,
|          and archive queries.
|
| Input:   sel_dialog  - Pointer to dialog table to process.
|
| Returns: RC_OK             - Successful
|          RC_NOT_IN_SESSION - Client is not in session with the server
|          RC_SESSION_FAILED - A dsm api call failed stopping execution
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
int16 perform_query_get(sel_dialog, parm_get_type)
   dialog *sel_dialog;
   uint16 parm_get_type;
#else
int16 perform_query_get(dialog *sel_dialog,uint16 parm_get_type)
#endif
{

uint16     i;
char       *desc;
bool_t     done = bFalse;
dialog    *dlg;
dsmObjName objName;      /* Big object for some compiler stacks......*/
int16      rc;
uint8      active_type;
bool_t     fast_active;
qryArchiveData     archiveData;
qryRespArchiveData respArchive;
qryBackupData      backupData;
qryRespBackupData  respBackup;
qryABackupData     abackupData;
qryARespBackupData respABackup;
char          owner[DSM_MAX_OWNER_LENGTH];
dsmQueryType  query_type;
dsmQueryBuff  *query_buff;
DataBlk       qData;
uint16        get_type;
uint64        qry_item64;
char          *s;
char          splat[2];
GetInfo       *llHead = NULL, *llTail = NULL, *ll = NULL;
smpObjInfo    *objInfo;
bool_t        details = bFalse;
char          t[50], t2[50], t4[50];
uint64        t64;
bool_t        mountWait = bFalse;
char          go[5];

strcpy(owner, "");
strcpy(splat, "*");

if (!dsmHandle)
{
   printf("You must signon before performing this action.\n");
   return(0);
}

if (!(s = malloc(sizeof(dsmObjName))))
{
   printf("Memory alloc failed in perform_query_get.\n");
   return RC_NO_MEMORY;
}

get_type = parm_get_type;    /* Put into a value we can modify. */

memset(&objName,0x00,sizeof(dsmObjName));

/*-----------------------------------------------------------------------
|  First pull all values out of the passed dialog for our use.
.-----------------------------------------------------------------------*/
i = 0;
dlg = &sel_dialog[i];
while (dlg->item_type != DSMAPI_END)
{
   switch (dlg->item_type)
   {
      case DSMAPI_FILESPACE :
         if ((strlen(dlg->item_buff) > DSM_MAX_FSNAME_LENGTH)
            || !*dlg->item_buff)
         {
            strcpy(objName.fs,"*");
         }
         else strcpy(objName.fs,dlg->item_buff);
         break;
      case DSMAPI_HIGHLEVEL :
         if ((strlen(dlg->item_buff) > DSM_MAX_HL_LENGTH)
            || !*dlg->item_buff)
         {
           objName.hl[0] = '*';  /* don't use delimiter - optional */
           objName.hl[1] = 0x00;
         }
          else strcpy(objName.hl,dlg->item_buff);
          break;
      case DSMAPI_LOWLEVEL :
         if ((strlen(dlg->item_buff) > DSM_MAX_LL_LENGTH)
            || !*dlg->item_buff)
         {
           objName.ll[0] = dsmSessInfo.hldelim;
           objName.ll[1] = '*';
           objName.ll[2] = 0x00;
         }
         else strcpy(objName.ll,dlg->item_buff);
         break;
      case DSMAPI_OBJOWNER :
         if ((strlen(dlg->item_buff) > DSM_MAX_OWNER_LENGTH)
            || !*dlg->item_buff)
         {
            strcpy(owner,"");
         }
         else strcpy(owner,dlg->item_buff);
         break;
      case DSMAPI_OBJTYPE :
         switch (*dlg->item_buff)
         {
            case 'F':
            case 'f':
               objName.objType = DSM_OBJ_FILE; break;
            case 'D':
            case 'd':
               objName.objType = DSM_OBJ_DIRECTORY; break;
            case 'A':
            case 'a':
               objName.objType = DSM_OBJ_ANY_TYPE; break;
            default :
               objName.objType = DSM_OBJ_FILE;
         }
         break;
      case DSMAPI_ACTIVETYPE :
         switch (*dlg->item_buff)
         {
            case 'A':
            case 'a':
               active_type = DSM_ACTIVE; break;
            case 'I':
            case 'i':
               active_type = DSM_INACTIVE; break;
            case 'B' :
            case 'b' :
               active_type = DSM_ANY_MATCH; break;
            default :
               active_type = DSM_ANY_MATCH;
         }
         break;
      case DSMAPI_DESCRIPTION :
         if ((strlen(dlg->item_buff) > DSM_MAX_DESCR_LENGTH)
            || !*dlg->item_buff)
         {
            desc = splat;
         }
         else desc = dlg->item_buff;
         break;
      case DSMAPI_FASTACTIVE :
         switch (*dlg->item_buff)
         {
            case 'Y':
            case 'y':
               fast_active = bTrue;
               /* change get_type only for Query Backup */
               if (get_type == Backup_Get)
                   get_type = ABackup_Get;
               break;
            case 'N':
            case 'n':
               fast_active = bFalse; break;
            default :
               fast_active = bFalse;
         }
      case DSMAPI_DETAILS :
         switch (*dlg->item_buff)
         {
            case 'Y':
            case 'y':
               details = bTrue;
               break;
            case 'N':
            case 'n':
               details = bFalse;
               break;
            default :
               details = pref.qry_details;
         }
         break;
      case DSMAPI_MNTWAIT :
         switch (*dlg->item_buff)
         {
            case 'Y':
            case 'y':
               mountWait = bTrue;              break;
            case 'N':
            case 'n':
               mountWait = bFalse;             break;
            default :
               mountWait = bFalse;             break;
         }
         break;
      default :
         ;
   }
   i++;
   dlg = &sel_dialog[i];
}

/* Only one object type allowed for archive, so set it here. */
if (   (get_type == Archive_Get)
    || (get_type == Archive_Delete)
    || (get_type == Retrieve_Get))
{
   objName.objType = DSM_OBJ_FILE;
}
/*----------------------------------------------------------------------
|  Fill in the proper structures for the type of query being performed.
.-----------------------------------------------------------------------*/
switch (get_type)
{
   case ABackup_Get :
      abackupData.stVersion = qryABackupDataVersion;
      abackupData.objName   = &objName;

      query_type = qtBackupActive;
      query_buff = (dsmQueryBuff *)&abackupData;

      qData.stVersion = DataBlkVersion;
      qData.bufferLen = sizeof(qryARespBackupData);
      qData.bufferPtr = (char *)&respABackup;
      respABackup.stVersion = qryARespBackupDataVersion;
      break;

   case Backup_Get  :
   case Restore_Get :
   case Backup_Delete :
      backupData.stVersion = qryBackupDataVersion;
      backupData.objName   = &objName;
      backupData.owner     = owner;
      backupData.objState  = active_type;

      query_type = qtBackup;
      query_buff = (dsmQueryBuff *)&backupData;

      qData.stVersion = DataBlkVersion;
      qData.bufferLen = sizeof(qryRespBackupData);
      qData.bufferPtr = (char *)&respBackup;
      respBackup.stVersion = qryRespBackupDataVersion;;
      break;

   case Archive_Get  :
   case Retrieve_Get :
   case Archive_Delete :
      archiveData.stVersion = qryArchiveDataVersion;
      archiveData.objName   = &objName;
      archiveData.owner     = owner;

      /* Note this needs to be changed when dialog handles date input */
      archiveData.insDateLowerBound.year   = DATE_MINUS_INFINITE;
      archiveData.insDateLowerBound.month  = 0;
      archiveData.insDateLowerBound.day    = 0;
      archiveData.insDateLowerBound.hour   = 0;
      archiveData.insDateLowerBound.minute = 0;
      archiveData.insDateLowerBound.second = 0;

      archiveData.insDateUpperBound.year   = DATE_PLUS_INFINITE;
      archiveData.insDateUpperBound.month  = 0;
      archiveData.insDateUpperBound.day    = 0;
      archiveData.insDateUpperBound.hour   = 0;
      archiveData.insDateUpperBound.minute = 0;
      archiveData.insDateUpperBound.second = 0;

      archiveData.expDateLowerBound.year   = DATE_MINUS_INFINITE;
      archiveData.expDateLowerBound.month  = 0;
      archiveData.expDateLowerBound.day    = 0;
      archiveData.expDateLowerBound.hour   = 0;
      archiveData.expDateLowerBound.minute = 0;
      archiveData.expDateLowerBound.second = 0;

      archiveData.expDateUpperBound.year   = DATE_PLUS_INFINITE;
      archiveData.expDateUpperBound.month  = 0;
      archiveData.expDateUpperBound.day    = 0;
      archiveData.expDateUpperBound.hour   = 0;
      archiveData.expDateUpperBound.minute = 0;
      archiveData.expDateUpperBound.second = 0;

      archiveData.descr = desc;
      query_type = qtArchive;
      query_buff = (dsmQueryBuff *)&archiveData;

      qData.stVersion = DataBlkVersion;
      qData.bufferLen = sizeof(qryRespArchiveData);
      qData.bufferPtr = (char *)&respArchive;
      respArchive.stVersion = qryRespArchiveDataVersion;
      break;
   default : ;
}
/*----------------------------------------------------------------------
|  Now issue the begin query call, and if no errors issue get next calls
|  until all query information has been processed.  For each query response
|  output the proper info to the screen or save it for later processing.
.-----------------------------------------------------------------------*/
if ((rc = dsmBeginQuery(dsmHandle,query_type,query_buff)))
{
   printf("*** dsmBeginQuery failed: \n");
   rcApiOut(rc);
   rc = (RC_SESSION_FAILED);
}
else
{
   done = bFalse;
   qry_item64.hi = 0; qry_item64.lo = 0;
   while (!done)
   {
      rc = dsmGetNextQObj(dsmHandle,&qData);
      if ((   (rc == DSM_RC_OK)
           || (rc == DSM_RC_MORE_DATA)
           || (rc == DSM_RC_FINISHED))
          && qData.numBytes)
      {
         switch(get_type)
         {
            /*-----------------------------
            |  Display archive query results
            .------------------------------*/
            case Archive_Get :
               qry_item64 = Incr64(qry_item64,1);
               if (details) printf("\n");
               I64toCh(qry_item64,t4,10);  format_number(t4,t);
               printf("Item %s: ",t);                     /* Item number */
               printf("%s%s%s\n",respArchive.objName.fs,  /* Name        */
                                 respArchive.objName.hl,
                                 respArchive.objName.ll);

               if (!details) break;

               QryPrintObjType(respArchive.objName.objType);
               printf("   Desc: %s\n",respArchive.descr);
               QryDateToStr(s,respArchive.insDate);
               printf("   Insert date: %s\n",s);
               QryDateToStr(s,respArchive.expDate);
               printf("   Expiration date: %s\n",s);
               printf("   Owner: %s\n",respArchive.owner);
               printf("   Restore order: %lu-%lu\n",
                               respArchive.restoreOrder.hi,
                               respArchive.restoreOrder.lo);
               printf("   Object id: %lu-%lu\n",respArchive.objId.hi,
                                             respArchive.objId.lo);
               printf("   Copy group: %lu\n",respArchive.copyGroup);
               QryPrintMediaClass(respArchive.mediaClass);
               QryPrintObjInfo(respArchive.objInfolen,
                               respArchive.objInfo);
               printf("   Mgmt class: %s\n",respArchive.mcName);
               printf("Press any key to continue\n");
               ourGetS(go);
               break;
            /*-----------------------------
            |  Display backup query results
            .------------------------------*/
            case Backup_Get  :
               qry_item64 = Incr64(qry_item64,1);
               if (details) printf("\n");
               I64toCh(qry_item64,t4,10);  format_number(t4,t);
               printf("Item %s: ",t);                     /* Item number */
               printf("%s%s%s\n",respBackup.objName.fs,
                                 respBackup.objName.hl,
                                 respBackup.objName.ll);

               if (!details) break;

               QryPrintObjType(respBackup.objName.objType);
               QryPrintObjState(respBackup.objState);
               QryDateToStr(s,respBackup.insDate);
               printf("   Insert date: %s\n",s);
               QryDateToStr(s,respBackup.expDate);
               printf("   Expiration date: %s\n",s);
               printf("   Owner: %s\n",respBackup.owner);
               printf("   Restore order: %lu-%lu\n",
                                 respBackup.restoreOrder.hi,
                                 respBackup.restoreOrder.lo);
               printf("   Object id: %lu-%lu\n",respBackup.objId.hi,
                                             respBackup.objId.lo);
               printf("   Copy group: %lu\n",respBackup.copyGroup);
               QryPrintMediaClass(respBackup.mediaClass);
               QryPrintObjInfo(respBackup.objInfolen,
                               respBackup.objInfo);
               printf("   Mgmt class: %s\n",respBackup.mcName);
               break;
            /*------------------------------------
            |  Display active backup query results
            .------------------------------------*/
            case ABackup_Get :
               qry_item64 = Incr64(qry_item64,1);
               if (details) printf("\n");
               I64toCh(qry_item64,t4,10);  format_number(t4,t);
               printf("Item %s: ",t);                     /* Item number */
               printf("%s%s%s\n",respABackup.objName.fs,
                                 respABackup.objName.hl,
                                 respABackup.objName.ll);

               if (!details) break;

               QryPrintObjType(respABackup.objName.objType);
               QryDateToStr(s,respABackup.insDate);
               printf("   Insert date: %s\n",s);
               printf("   Owner: %s\n",respABackup.owner);
               printf("   Copy group: %lu\n",respABackup.copyGroup);
               QryPrintObjInfo(respABackup.objInfolen,
                               respABackup.objInfo);
               printf("   Mgmt class: %s\n",respABackup.mcName);
               break;
            /*-----------------------------------------------
            |  Save key data from query for restore or delete
            .------------------------------------------------*/
            case Restore_Get :
            case Backup_Delete :

               /*---------------------------------------
               |  Only can delete active backup files!
               .--------------------------------------*/
               if ((get_type == Backup_Delete)
                   && (respBackup.objState != DSM_ACTIVE))
                  break;
               qry_item64 = Incr64(qry_item64,1);
               rc = QryAddGetLlist(get_type,
                                  qry_item64,
                                  &llHead,
                                  &llTail,
                                  (char *)&respBackup);
               if (rc)
               {
                  QryFreeGetLlist(&llHead,
                                 &llTail);
                  switch (get_type)
                  {
                     case Restore_Get :
                        printf("Restore failed during the backup query.\n");
                        break;
                     case Backup_Delete :
                     default :
                        printf("Backup delete failed during the query.\n");
                  }

               }
               break;
            /*------------------------------------------------
            |  Save key data from query for retrieve or delete
            .------------------------------------------------*/
            case Retrieve_Get :
            case Archive_Delete :
               qry_item64 = Incr64(qry_item64,1);
               rc = QryAddGetLlist(get_type,
                                  qry_item64,
                                  &llHead,
                                  &llTail,
                                  (char *)&respArchive);
               if (rc)
               {
               QryFreeGetLlist(&llHead,
                              &llTail);
                  switch (get_type)
                  {
                     case Retrieve_Get :
                        printf("Retrieve failed during the backup query.\n");
                        break;
                     case Archive_Delete :
                     default :
                        printf("Archive delete failed during the query.\n");
                  }
               }
               break;
            /*------------------
            |  Default catch all
            .-------------------*/
            default : ;
         }
      }
      else
      {
         done = bTrue;
         if (rc != DSM_RC_FINISHED)
         {
            printf("*** dsmGetNextQObj failed: \n");
            rcApiOut(rc);
         }
      if (rc == DSM_RC_FINISHED) done = bTrue;
      }

   }

   rc = dsmEndQuery(dsmHandle);
}

if (   (get_type == Restore_Get)
    || (get_type == Retrieve_Get))
{
   rc = QryDoGet(get_type,llHead,llTail,qry_item64,mountWait);
   QryFreeGetLlist(&llHead,&llTail);
}

if (   (get_type == Backup_Delete)
    || (get_type == Archive_Delete))
{
   rc = QryDoDelete(get_type,llHead,llTail,qry_item64);
   QryFreeGetLlist(&llHead,&llTail);
}

return rc;
}


/*----------------------------------------------------------------------+
| Name:    QryDateToStr()
|
| Action:  A dumb, non-NLS enabled date to string conversion routine.
|          Just put the passed date into the passed string in the form
|          yyyy/mm/dd hh:mm:ss
|
| Input:   s - Pointer to a buffer big enough to hold the converted date
|          date - Date to convert.
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void QryDateToStr(s, date)
   char    *s;
   dsmDate date;
#else
static void QryDateToStr(char *s,dsmDate date)
#endif
{
   sprintf(s,"%i/%i/%i %i:%i:%i",
                      date.year,
               (int16)date.month,
               (int16)date.day,
               (int16)date.hour,
               (int16)date.minute,
               (int16)date.second);
}

/*----------------------------------------------------------------------+
| Name:    QryAddGetLlist()
|
| Action:  Add the passed structure to the passed linked list.  Handle
|          the standard situation of new list or existing list.  Note
|          list structure is malloc'd and fields within the list are also
|          malloc'd.  These will all be freed by the corresponding free
|          llist routine.  A more efficient method would be to use a
|          memory pool manager package for these small memory allocs, but
|          then this is just a sample app. for the api!
|
| Input:
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryAddGetLlist(get_type, qry_item64, llHead, llTail, buff)
   uint16  get_type;
   uint64  qry_item64;
   GetInfo **llHead;
   GetInfo **llTail;
   char    *buff;
#else
static int16 QryAddGetLlist(uint16 get_type,
                            uint64  qry_item64,
                            GetInfo **llHead,
                            GetInfo **llTail,
                            char *buff)
#endif
{
   qryRespArchiveData *respA;
   qryRespBackupData  *respB;
   GetInfo           *new = NULL, *ll = NULL;
   dsmObjNameP        objName;
   ObjID              objId;
   uint32             copyGroup;
   uint64             restoreOrder;
   uint8              mediaClass;
   uint16             objInfolen;
   char               *objInfo;
   smpObjInfo         *tobjInfo;
   char               t[50], t4[50];



   switch (get_type)
   {
      case (Restore_Get) :
      case (Backup_Delete) :
         respB = (qryRespBackupData *)buff;
         objName.fs = respB->objName.fs;
         objName.hl = respB->objName.hl;
         objName.ll = respB->objName.ll;
         objName.objType = respB->objName.objType;
         objId.hi   = respB->objId.hi;
         objId.lo   = respB->objId.lo;
         copyGroup  = respB->copyGroup;
         restoreOrder.hi = respB->restoreOrder.hi;
         restoreOrder.lo = respB->restoreOrder.lo;
         mediaClass      = respB->mediaClass;
         objInfolen      = respB->objInfolen;
         objInfo         = respB->objInfo;
         break;
      case (Retrieve_Get) :
      case (Archive_Delete) :
         respA = (qryRespArchiveData *)buff;
         objName.fs = respA->objName.fs;
         objName.hl = respA->objName.hl;
         objName.ll = respA->objName.ll;
         objName.objType = respA->objName.objType;
         objId.hi   = respA->objId.hi;
         objId.lo   = respA->objId.lo;
         copyGroup  = respA->copyGroup;
         restoreOrder.hi = respA->restoreOrder.hi;
         restoreOrder.lo = respA->restoreOrder.lo;
         mediaClass      = respA->mediaClass;
         objInfolen      = respA->objInfolen;
         objInfo         = respA->objInfo;
         break;
      default : ;
   }

   if (!(new = (GetInfo *)malloc(sizeof(GetInfo))))
     return RC_NO_MEMORY;

   memset(new,0x00,sizeof(GetInfo));
   /*--------------------------------------------------------------------
   | First get memory areas for other parts of new item.
   .--------------------------------------------------------------------*/
   if (!(new->objName = malloc(sizeof(dsmObjNameP))))
   {
      QryFreeGetLlistItem(&new); return RC_NO_MEMORY;
   }

   if (!(new->objName->fs = malloc(strlen(objName.fs) + 1)))
   {
      QryFreeGetLlistItem(&new); return RC_NO_MEMORY;
   }

   if (!(new->objName->hl = malloc(strlen(objName.hl) + 1)))
   {
      QryFreeGetLlistItem(&new); return RC_NO_MEMORY;
   }

   if (!(new->objName->ll = malloc(strlen(objName.ll) + 1)))
   {
      QryFreeGetLlistItem(&new); return RC_NO_MEMORY;
   }

   if (!(new->objInfo = malloc(objInfolen)))
   {
      QryFreeGetLlistItem(&new); return RC_NO_MEMORY;
   }

   if (!*llHead)
   {
     *llHead = new;
     *llTail = new;
   }
   else
   {
      ll = *llTail;
      ll->next = new;
      *llTail  = new;
   }

   new->next = NULL;
   new->item64.hi = qry_item64.hi;
   new->item64.lo = qry_item64.lo;
   strcpy(new->objName->fs,objName.fs);
   strcpy(new->objName->hl,objName.hl);
   strcpy(new->objName->ll,objName.ll);
   new->objName->objType = objName.objType;

   new->objId.hi = objId.hi;
   new->objId.lo = objId.lo;

   new->restoreOrder.hi = restoreOrder.hi;
   new->restoreOrder.lo = restoreOrder.lo;

   new->copyGroup  = copyGroup;
   new->mediaClass = mediaClass;

   new->objInfolen = objInfolen;
   memcpy(new->objInfo,objInfo,objInfolen);

   if (pref.verbose)
   {
      tobjInfo = (smpObjInfo *)objInfo;
      I64toCh(tobjInfo->size,t4,10);  format_number(t4,t);
      printf("Object info for %s%s%s :\n",objName.fs,objName.hl,objName.ll);
      printf("   Size of object : %s\n",t);
      printf("   Seed string ==>%s<==\n",tobjInfo->seed);
   }

   return 0;
}

/*----------------------------------------------------------------------+
| Name:    QryDoGet()
|
| Action:  A linked list of items to restore has been created.  Process
|          this list to create the proper list for the GetData calls.
|
| Input:
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryDoGet(get_type, llHead, llTail, items64, mountWait)
   uint16 get_type;
   GetInfo *llHead;
   GetInfo *llTail;
   uint64  items64;
   bool_t  mountWait;
#else
static int16 QryDoGet(uint16 get_type,
                      GetInfo *llHead,
                      GetInfo *llTail,
                      uint64  items64,
                      bool_t  mountWait)
#endif
{
   GetInfo *ll = NULL, *llStart = NULL; /* Pointers to objects in the list */
   char    *rest_ibuff, *ibuff_cursor;  /* Object id buffer and index in buff*/
   char    *part_pbuff, *pbuff_cursor;  /* Partial object pointers */
   uint32  loop_item;                   /* Loop counter */
   ObjID   *objId;                      /* Unique id for object */
   PartialObjData *partialObjData;      /* Pointer to partial object data */
   int16  rc,i;
   char   buff[50];
   char   t[50], t2[50], t4[50];
   uint64 offset,length;
   bool_t   partialObjRequested = bFalse;

   if (!llHead) return 0;  /* No items in list to restore */

   /* Get buffer to put object ids in to be restored */
   if (!(rest_ibuff = malloc((pref.max_restore_items * sizeof(ObjID))+1)))
        return RC_NO_MEMORY;

   /* Get buffer to put partial object data into */
   if (!(part_pbuff = malloc((pref.max_restore_items * sizeof(PartialObjData))+1)))
        return RC_NO_MEMORY;
   pbuff_cursor = part_pbuff;

   ll = llHead; llStart = llHead;  /* Init list traversal items */
   ibuff_cursor = rest_ibuff;
   loop_item = 0;

   I64toCh(items64,t4,10);               format_number(t4,t);
   sprintf(t4,"%lu",pref.sz_recv_buff);  format_number(t4,t2);

   switch(get_type)
   {
      case (Restore_Get) :
         printf("Starting restore of %s objects in buffers %s bytes long.\n",
           t,t2);
         break;
      case (Retrieve_Get) :
         printf("Starting retrieve of %s objects in buffers %s bytes long.\n",
           t,t2);
         break;
      default : ;
   }

/*   printf("Press enter to proceed.\n");
   ourGetS(buff); */


   printf("Enter 0 for offset and length for normal object retrieve\n");

   for(;;)
   {
      /*------------------------------------------------------------------
      | Bump through list creating list of object ids to restore until
      | list is full or no more items to put in it.  32bit test was
      .-----------------------------------------------------------------*/
      if ((loop_item < pref.max_restore_items) && ll)
      {
         /* Set object id list */
         objId = (ObjID *)ibuff_cursor;    /* Cast buffer to proper type */
         objId->hi = ll->objId.hi;         /* Load up hi 4 bytes...      */
         objId->lo = ll->objId.lo;         /* And lo 4 bytes.            */
         /* Set partial object data list */
         partialObjData = (PartialObjData *)pbuff_cursor;
         printf("Enter partial object offset for object %i: ",
                 loop_item + 1);
         ourGetS(buff);
         offset = ChtoI64(buff,10);
         if (!partialObjRequested && (offset.hi || offset.lo))
            partialObjRequested = bTrue;
         partialObjData->partialObjOffset.hi = offset.hi;
         partialObjData->partialObjOffset.lo = offset.lo;
         printf("Enter partial object length for object %i: ",
                 loop_item + 1);
         ourGetS(buff);
         length = ChtoI64(buff,10);
         if (!partialObjRequested && (length.hi || length.lo))
            partialObjRequested = bTrue;
         partialObjData->partialObjLength.hi = length.hi;
         partialObjData->partialObjLength.lo = length.lo;
         /* Increment cursors and loop counters */
         pbuff_cursor += sizeof(PartialObjData);
         ibuff_cursor += sizeof(ObjID);    /* Point to next location.    */
         loop_item++;                      /* Item in list.              */
         ll = ll->next;                    /* Go to next item if any.    */
      }
      else
{

         /*--------------------------------------------------------------
         | See if there are items to actually restore.  If so do so else
         | done with the restore process.
         .--------------------------------------------------------------*/
         if (loop_item)
         {
         if (partialObjRequested) /* If partial object request */
            rc = QryDoRestoreObjs(get_type,llStart,rest_ibuff,loop_item,
                                  mountWait, part_pbuff);
         else                     /* else not a partial object request */
            rc = QryDoRestoreObjs(get_type,llStart,rest_ibuff,loop_item,
                                  mountWait, NULL);
         }
         else break;  /* ended on an even boundry! */

         /*--------------------------------------------------------------
         | Now see if there are more items to process, if so do so.
         .---------------------------------------------------------------*/
         if (ll)
         {
            loop_item = 0;               /* Start new cycle */
            ibuff_cursor = rest_ibuff;   /* Point for front of buff */
            pbuff_cursor = part_pbuff;   /* Point for front of buff */
            llStart = ll;                /* New cursor into linked list */
         }
         else break;
      }
   }
   free(rest_ibuff);
   free(part_pbuff);
   return 0;
}

/*----------------------------------------------------------------------+
| Name:    QryDoRestoreObjs
|
| Action:  We now have a list of object id's to restore.  Create the
|          calls to receive the data and validate it as it is received.
|
| Input:   llStart : Pointer to the linked list item that is first to be
|                    restored.  Each item in this list has a 1-1
|                    correspondance with the rest_ibuff list of obj ids.
|
|          rest_ibuff : Restore index buffer with object id's to resore.
|                       This is input to the api getdata calls.
|
|          items : Number of items to process.  Not really needed but
|                  good info to have for human output.
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryDoRestoreObjs(get_type, llStart, rest_ibuff, items, mountWait, part_pbuff)
   uint16  get_type;
   GetInfo *llStart;
   char    *rest_ibuff;
   uint32  items;
   bool_t  mountWait;
   char    *part_pbuff;
#else
static int16 QryDoRestoreObjs(uint16  get_type,
                              GetInfo *llStart,
                              char    *rest_ibuff,
                              uint32  items,
                              bool_t  mountWait,
                              char    *part_pbuff)
#endif
{
   int16    rc, ver_rc;
   uint32   i;
   char     *get_buff;
   ObjID    *objId;
   DataBlk  dataBlk;
   GetInfo  *ll = NULL;
   bool_t   done,done2;
   bool_t   validate_data = bFalse;
   dsmGetList getList;
   smpObjInfo *objInfo;
   char       *seed_buff;
   seedValidate seedV;
   char       t[30];

   if (!(get_buff = malloc(pref.sz_recv_buff)))
   {
      printf("Memory alloc error on get_guff.\n");
      return RC_NO_MEMORY;
   }

   getList.stVersion = dsmGetListPORVersion;
   getList.numObjId  = items;
   getList.objId     = (ObjID *)rest_ibuff;
   getList.partialObjData = (PartialObjData *)part_pbuff;
   /*-----------------------------------------------------------------------
   |  Issue the BeginGetData call with the list of ojbect id's we will be
   |  obtaining from the server.  This is then followed with a get for
   |  each separate object id and its associated data.  As data is
   |  received it will be verified against the actual data sent by looking
   |  at the objInfo field to calculate correctness.
   .-----------------------------------------------------------------------*/
   switch(get_type)
   {
      case (Restore_Get) :
        rc = dsmBeginGetData(dsmHandle,mountWait,gtBackup,&getList);
        break;
      case (Retrieve_Get) :
        rc = dsmBeginGetData(dsmHandle,mountWait,gtArchive,&getList);
        break;
      default : ;
   }

   if (rc)
   {
      printf("*** dsmBeginGetData failed: \n");
      rcApiOut(rc);
      return rc;
   }

   ll = llStart;
   objId = (ObjID *)rest_ibuff;
   if (!(seedV.seed = malloc(pref.sz_send_buff)))
   {
      printf("Malloc failed for validation seed buff.\n");
      return RC_NO_MEMORY;
   }
   seedV.seed_string_size = pref.sz_send_buff ;

   /*-----------------------------------------------------------------------
   |  Now go get each object from the list for restore or retrieve purposes.
   |  Each object will be verified with data stored in the info field when
   |  it was sent to insure integrity of the test.
   .-----------------------------------------------------------------------*/
   for (i=0;(i < items) && ll; i++)
   {
      dataBlk.stVersion = DataBlkVersion;
      dataBlk.bufferLen = pref.sz_recv_buff;
      dataBlk.numBytes  = 0;
      dataBlk.bufferPtr = get_buff;

      I64toCh(ll->item64,t,10);

      if (get_type == Restore_Get)
        printf("Restoring object %s: %s%s%s :\n",t,
                ll->objName->fs,ll->objName->hl,ll->objName->ll);
      else  /* Retrieve_Get */
        printf("Retrieving object %s: %s%s%s :\n",t,
                ll->objName->fs,ll->objName->hl,ll->objName->ll);

      /*--------------------------------------------------------------------
      |  See if this was an object the sample api sent to the server.  If
      |  so init structures with object attributes for data validation
      |  checks.  If not our data then bypass data verification checks.
      .--------------------------------------------------------------------*/
      validate_data = bFalse;
      if (ll->objInfolen >= sizeof(smpObjInfo))   /* Even big enough for us?*/
      {
         objInfo = (smpObjInfo *)ll->objInfo;
         if (!(strcmp(api_eyecatcher,objInfo->eyecatcher)))
         {
            validate_data = bTrue;
            seedV.seed_len = strlen(seedV.seed);
            seedV.orig_sz.hi = objInfo->size.hi;
            seedV.orig_sz.lo = objInfo->size.lo;
            seedV.total_sz64.hi = 0;
            seedV.total_sz64.lo = 0;
            seedV.seed_cursor = seedV.seed;
            seedV.first_time = bTrue;

            QryMakeSeed(&seedV,objInfo->seed);
         }
      }

      rc = dsmGetObj(dsmHandle,objId,&dataBlk);

      /*--------------------------------------------------------------------
      |  Go check the results of the above get, and loop through getdata
      |  calls as needed until all data is obtained from the server.  Note
      |  that each buffer received will be validated to insure data
      |  integrity if the data in the objInfo field was generated by the
      |  sample api program.  Otherwise data validation is skipped.
      .--------------------------------------------------------------------*/
      done = bFalse;
      while(!done)
      {
         if (   (rc == DSM_RC_MORE_DATA)
             || (rc == DSM_RC_FINISHED))
         {

            /* validate data here */
            if (validate_data)
            {
               ver_rc = Qry_Validate_Data(&dataBlk,&seedV,i,part_pbuff);
            }

            if (rc == DSM_RC_MORE_DATA)
            {
               dataBlk.numBytes = 0;
               rc = dsmGetData(dsmHandle,&dataBlk);
            }
            else
            {
               done = bTrue;
               if (validate_data)
               {
                  if (Cmp64(seedV.total_sz64,seedV.orig_sz) != EQUAL)
                  {
                     if (!ver_rc)
                        printf("Object content verified\n");
                     I64toCh(seedV.total_sz64,t,10);
                     printf(" Object length returned: %s\n", t );
                     I64toCh(seedV.orig_sz,t,10);
                     printf(" Original object length: %s\n", t );
                     }
                  else
                     printf("Object content and length verified!\n");
               }
            else printf(" Done!\n");
            }
         }
         else
         {
            printf("*** dsmGetObj or dsmGetData failed: \n");
            rcApiOut(rc);
            done = bTrue;
         }
      } /* while */
      rc = dsmEndGetObj(dsmHandle);

      ll = ll->next;
      objId++;  /* Next object to process. */
   } /* for */


   rc = dsmEndGetData(dsmHandle);
   free(get_buff);
return 0;
}


/*----------------------------------------------------------------------+
| Name:    QryDoDelete()
|
| Action:  A linked list of items to delete has been created.  Process
|          this list within transaction boundaries to delete the objects.
|
| Input:
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryDoDelete(delete_type, llHead, llTail, items64)
   uint16  delete_type;
   GetInfo *llHead;
   GetInfo *llTail;
   uint64  items64;
#else
static int16 QryDoDelete(uint16 delete_type,
                         GetInfo *llHead,
                         GetInfo *llTail,
                         uint64  items64)
#endif
{
   GetInfo *ll = NULL;                  /* Pointers to objects in the list */
   ObjID   *objId;                      /* Unique id for object */
   int16  rc;
   char   buff[50];
   dsmDelInfo delInfo;
   dsmDelType delType;
   bool_t  in_txn = bFalse;
   bool_t  new_txn_needed = bFalse;
   uint16  txn_reason;
   uint32  txn_objs;
   dsmObjName objName;
   char t[50], t2[50], t3[50], t4[50];

   if (!llHead) return 0;  /* No items in list to delete */

   ll = llHead;                    /* Init list traversal items */

   I64toCh(items64,t4,10); format_number(t4,t);
   printf("Starting delete of %s objects.\nPress enter to proceed.\n",t);
   ourGetS(buff);

   /*----------------------------------------------------------------------+
   | Bump through the linked list deleting objects within the bounds of
   | multiple object transactions.
   .----------------------------------------------------------------------*/
   while (ll)
   {
      I64toCh(ll->item64,t4,10);  format_number(t4,t2);
      printf("Deleting object %s of %s: %s%s%s\n",
             t2,t,ll->objName->fs,ll->objName->hl,ll->objName->ll);

      if (!in_txn)
      {
         if ((rc = QryBeginTxn(dsmHandle,     /* DSM API sess handle         */
                              &txn_objs)))    /* Objects in this txn.        */

         {
            printf("*** dsmBeginTxn(1) failed: \n");
            rcApiOut(rc);
            rc = (RC_SESSION_FAILED);
            break;
         }
         /*-----------------------------------------------------------------.
         | If we get here we are in session and within the bounds of a txn
         | and all of our transaction counters, etc., are initialized.
         .-----------------------------------------------------------------*/
         new_txn_needed = bFalse;
         in_txn = bTrue;
      }

      /*--------------------------------------------------------------------.
      | Are we within the max number of objects for a transaction?
      .--------------------------------------------------------------------*/
      if (txn_objs >= dsmSessInfo.maxObjPerTxn)
      {
         new_txn_needed = bTrue;
         printf("End txn: Current txn objects (%lu) >= max txn objects(%u).\n",
               txn_objs,dsmSessInfo.maxObjPerTxn);
      }

      /*--------------------------------------------------------------------.
      | See if a new txn is needed.  If so end current txn and start a new
      | one.
      .--------------------------------------------------------------------*/
      if (new_txn_needed)
      {
         txn_reason = 0;
         rc = dsmEndTxn(dsmHandle,        /* DSM API sess handle         */
                        DSM_VOTE_COMMIT,  /* Commit transaction.         */
                        &txn_reason);     /* Reason if txn aborted.      */
         if (rc || txn_reason)
         {
            printf("*** dsmEndTxn failed: \n");
            rcApiOut(rc);
            printf("   Reason = ");
            rcApiOut(txn_reason);
            rc = (RC_SESSION_FAILED);
            in_txn = bFalse;
            break;
         }

         if ((rc = QryBeginTxn(dsmHandle,     /* DSM API sess handle         */
                             &txn_objs)))    /* Objects in this txn.        */


         {
            printf("*** dsmBeginTxn(2) failed: \n");
            rcApiOut(rc);
            rc = (RC_SESSION_FAILED);
            break;
         }
         new_txn_needed = bFalse;
      }


      /*--------------------------------------------------------------------.
      | Now we can build and send the delete object api call.
      | We are in a transaction and are within the bounds of a
      | single or multiple transaction.
      .--------------------------------------------------------------------*/
      switch(delete_type)
      {
         case Backup_Delete :
            strcpy(objName.fs,ll->objName->fs);
            strcpy(objName.hl,ll->objName->hl);
            strcpy(objName.ll,ll->objName->ll);
            objName.objType = ll->objName->objType;
            delInfo.backInfo.stVersion = delBackVersion;
            delInfo.backInfo.objNameP  = &objName;
            delInfo.backInfo.copyGroup = ll->copyGroup;
            delType = dtBackup;
            break;
         case Archive_Delete :
            delInfo.archInfo.stVersion = delArchVersion;
            delInfo.archInfo.objId.hi  = ll->objId.hi;
            delInfo.archInfo.objId.lo  = ll->objId.lo;
            delType = dtArchive;
            break;
         default : ;
      }

      rc = dsmDeleteObj(dsmHandle,delType,delInfo);

      if (rc)
      {
         printf("*** dsmDeleteObj failed: \n");
         rcApiOut(rc);
         rc = (RC_SESSION_FAILED);
         break;
      }
      ll = ll->next;
      txn_objs++;
   }

   if (in_txn)
   {
      txn_reason = 0;
      rc = dsmEndTxn(dsmHandle,          /* DSM API sess handle         */
                    DSM_VOTE_COMMIT,     /* Commit transaction.         */
                    &txn_reason);        /* Reason if txn aborted.      */
      if (rc || txn_reason)
      {
         printf("*** dsmEndTxn(2) failed: \n");
         rcApiOut(rc);
         printf("   Reason = ");
         rcApiOut(txn_reason);
         rc = (RC_SESSION_FAILED);
      }

   }
}


/*-----------------------------------------------------------------------
|  Free the passed linked list and all associated storage
.-----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryFreeGetLlist(llHead, llTail)
   GetInfo **llHead;
   GetInfo **llTail;
#else
static int16 QryFreeGetLlist(GetInfo **llHead,
                            GetInfo **llTail)
#endif
{
   GetInfo *ll = NULL;

   ll = *llHead;
   while (ll)
   {
      *llHead = ll->next;
      QryFreeGetLlistItem(&ll);
      ll = *llHead;
   }
   *llHead = NULL;
   *llTail = NULL;
   return 0;
}

/*-----------------------------------------------------------------------
|  Free associated storage for the passed item.
.-----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryFreeGetLlistItem(item)
   GetInfo **item;
#else
static int16 QryFreeGetLlistItem(GetInfo **item)
#endif
{
   GetInfo *ll = NULL;

   ll = *item;
   free(ll->objName->fs);
   free(ll->objName->hl);
   free(ll->objName->ll);
   free(ll->objName);
   free(ll->objInfo);
   free(*item);
   *item = NULL;
   return 0;
}

/*----------------------------------------------------------------------+
| Name:    QryValidateData
|
| Action:  Check the data in the seed string against the returned buffer
|          to insure proper match.
|
| Input:
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 Qry_Validate_Data(dataBlk, seedV, item, part_pbuff)
   DataBlk       *dataBlk;
   seedValidate  *seedV;
   uint32        item;
   char          *part_pbuff;
#else
static int16 Qry_Validate_Data(DataBlk *dataBlk,seedValidate *seedV, uint32 item, char *part_pbuff)
#endif
{
   uint32   i,j;
   uint64   num_of_buffers, rmdr;
   PartialObjData  *t_buff;

   /* Find the partial object data for this object */
   if (part_pbuff && seedV->first_time )
      {
      t_buff = (PartialObjData *) 
        (part_pbuff + (sizeof(PartialObjData) * item)) ;

      num_of_buffers = Div64(t_buff->partialObjOffset,
                             Make64(pref.sz_send_buff),
                             &rmdr);
      seedV->seed_cursor = seedV->seed + rmdr.lo;
      seedV->first_time = bFalse ;
      }
   /* Verify each byte returned */
   for (i=0; i < dataBlk->numBytes; i++)
   {
      if ((seedV->seed_cursor - seedV->seed) == seedV->seed_string_size)
         seedV->seed_cursor = seedV->seed ; /* reset cursor */
      if (dataBlk->bufferPtr[i] == *seedV->seed_cursor)
         {
         seedV->total_sz64 = Incr64(seedV->total_sz64, (uint32) 1);
         seedV->seed_cursor += 1 ;
         }
      else
         {
         printf("** Error: Validation failed ** \n");
         return 1;
         }

   }
return 0;
}

/*----------------------------------------------------------------------+
| Name:    QryBeginTxn()
|
| Action:  Do a begin txn to the API as well as .............
|
| Input:
|
| Returns:
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryBeginTxn(dsmHandle, txn_objs)
   uint32  dsmHandle;
   uint32  *txn_objs;
#else
static int16 QryBeginTxn(uint32  dsmHandle,        /* DSM API sess handle  */
                         uint32  *txn_objs)        /* Objects in this txn. */
#endif

{
int16 rc;

rc = dsmBeginTxn(dsmHandle);
*txn_objs = 0;
return rc;
}

/*----------------------------------------------------------------------+
| Name:    QryPrintMediaClass
|
| Action:  Output text for the media class passed.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void QryPrintMediaClass(mediaClass)
   uint8 mediaClass;
#else
static void QryPrintMediaClass(uint8 mediaClass)
#endif
{
   printf("   Media class: ");
   switch (mediaClass)
   {
      case (MEDIA_FIXED) :
         printf("Fixed\n");       break;
      case (MEDIA_LIBRARY) :
         printf("Library\n");     break;
      case (MEDIA_NETWORK) :
         printf("Networked\n");   break;
      case (MEDIA_SHELF) :
         printf("Shelf\n");       break;
      case (MEDIA_OFFSITE) :
         printf("Offsite\n");     break;
      case (MEDIA_UNAVAILABLE) :
         printf("Unavailable\n"); break;
      default :
          printf("*** Unknown : %iu ***\n",(uint16) mediaClass);
   }
}

/*----------------------------------------------------------------------+
| Name:    QryPrintObjType
|
| Action:  Output text for the object type passed.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void QryPrintObjType(objType)
   uint8 objType;
#else
static void QryPrintObjType(uint8 objType)
#endif
{
   printf("   Object type: ");
   switch (objType)
   {
      case (DSM_OBJ_FILE) :
         printf("File\n");        break;
      case (DSM_OBJ_DIRECTORY) :
         printf("Directory\n");   break;
      default :
         printf("*** Unknown : %iu ***\n",(uint16) objType);
   }
}

/*----------------------------------------------------------------------+
| Name:    QryPrintObjInfo
|
| Action:  Output text for the object info.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void  QryPrintObjInfo(objInfolen, objInfo)
   uint16 objInfolen;
   char   *objInfo;
#else
static void  QryPrintObjInfo(uint16 objInfolen, char *objInfo)
#endif
{
   return;
}

/*----------------------------------------------------------------------+
| Name:    QryPrintObjState
|
| Action:  Output text for the object state passed.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void QryPrintObjState(objState)
   uint8 objState;
#else
static void QryPrintObjState(uint8 objState)
#endif
{
   printf("   Object state: ");
   switch (objState)
   {
      case (DSM_ACTIVE) :
         printf("Active\n");     break;
      case (DSM_INACTIVE) :
         printf("Inactive\n");   break;
      default :
         printf("*** Unknown : %iu ***\n",(uint16) objState);
   }
}

/*----------------------------------------------------------------------+
| Name:    QryPrintCopySer
|
| Action:  Output text for the copy serialization value passed.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void QryPrintCopySer(copySer)
   uint8 copySer;
#else
static void QryPrintCopySer(uint8 copySer)
#endif
{
   printf("   Copy Serialization: ");
   switch (copySer)
   {
      case (Copy_Serial_Static) :
         printf("Static\n");          break;
      case (Copy_Serial_Shared_Static) :
         printf("Shared Static\n");   break;
      case (Copy_Serial_Shared_Dynamic) :
         printf("Shared Dynamic\n");  break;
      case (Copy_Serial_Dynamic) :
         printf("Dynamic\n");         break;
      default :
         printf("*** Unknown : %iu ***\n",(uint16) copySer);
   }

}


/*----------------------------------------------------------------------+
| Name:    QryPrintCopyMode
|
| Action:  Output text for the copy mode value passed.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static void QryPrintCopyMode(copyMode)
   uint8 copyMode;
#else
static void QryPrintCopyMode(uint8 copyMode)
#endif
{
   printf("            Copy Mode: ");
   switch (copyMode)
   {
      case (Copy_Mode_Modified) :
         printf("Modified");        break;
      case (Copy_Mode_Absolute) :
         printf("Absolute");        break;
      default :
         printf("*** Unknown : %iu ***\n",(uint16) copyMode);
   }
}



/*----------------------------------------------------------------------+
| Name:    QryAddFsLlist()
|
| Action:  Add the passed structure to the passed linked list.  Handle
|          the standard situation of new list or existing list.  Note
|          list structure is malloc'd and fields within the list are also
|          malloc'd.  These will all be freed by the corresponding free
|          llist routine.  A more efficient method would be to use a
|          memory pool manager package for these small memory allocs, but
|          then this is just a sample app. for the api!
|
| Input:
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryAddFsLlist(qry_item, llHead, llTail, fsResp)
   uint32         qry_item;
   fsInfo         **llHead;
   fsInfo         **llTail;
   qryRespFSData  *fsResp;
#else
static int16 QryAddFsLlist(uint32  qry_item,
                           fsInfo **llHead,
                           fsInfo **llTail,
                           qryRespFSData *fsResp)
#endif

{
   fsInfo  *new = NULL, *ll;

   if (!(new = (fsInfo *)malloc(sizeof(fsInfo))))
     return RC_NO_MEMORY;

   memset(new,0x00,sizeof(fsInfo));
   /*--------------------------------------------------------------------
   | First get memory areas for other parts of new item.
   .--------------------------------------------------------------------*/
   if (!(new->fsName = malloc(strlen(fsResp->fsName) + 1)))
   {
      free(new); return RC_NO_MEMORY;
   }

   if (!*llHead)
   {
     *llHead = new;
     *llTail = new;
   }
   else
   {
      ll = *llTail;
      ll->next = new;
      *llTail  = new;
   }

   new->next = NULL;
   new->item = qry_item;
   strcpy(new->fsName,fsResp->fsName);

   if (pref.verbose)
   {
      printf("Inserted fs %s into the linked list.\n",new->fsName);
   }

   return 0;
}

/*-----------------------------------------------------------------------
|  Free the passed linked list and all associated storage
.-----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryFreeFsLlist(llHead, llTail)
   fsInfo **llHead;
   fsInfo **llTail;
#else
static int16 QryFreeFsLlist(fsInfo **llHead,
                            fsInfo **llTail)
#endif
{
   fsInfo *ll;

   ll = *llHead;
   while (ll)
   {
      *llHead = ll->next;
      free(ll->fsName);
      free(ll);
      ll = *llHead;
   }
   *llHead = NULL;
   *llTail = NULL;
   return 0;
}


/*----------------------------------------------------------------------+
| Name:    QryDoFsDelete()
|
| Action:  A linked list of fs's to delete has been created.  Process
|          this list to delete the filespaces.
|
| Input:
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryDoFsDelete(llHead, llTail, items)
   fsInfo  *llHead;
   fsInfo  *llTail;
   uint32  items;
#else
static int16 QryDoFsDelete(fsInfo *llHead,
                           fsInfo *llTail,
                           uint32  items)
#endif
{
   fsInfo *ll;                         /* Pointers to objects in the list */
   int16  rc;
   char   buff[50];
   char   t[50], t2[50], t4[50];

   if (!llHead) return 0;          /* No items in list to delete */

   ll = llHead;                    /* Init list traversal items */

   sprintf(t4,"%lu",items);  format_number(t4,t);
   printf("Starting delete of %s filespaces.\nPress enter to proceed.\n",t);
   ourGetS(buff);
   if (*buff)
   {
      printf("Filespace delete terminating at user request.\n");
      return 0;
   }

   /*----------------------------------------------------------------------+
   | Bump through the linked list deleting filespaces.
   .----------------------------------------------------------------------*/
   while (ll)
   {
      sprintf(t4,"%lu",ll->item); format_number(t4,t2);
      printf("\nDeleting filespace %s of %s: %s",t2,t,ll->fsName);
      rc = dsmDeleteFS(dsmHandle,ll->fsName,DSM_REPOS_ALL);
      if (rc)
      {
         printf("  FAILED!!! ");
         rcApiOut(rc);
      }
      else printf("  OK!\n");

   ll = ll->next;
   }

   return 0;
}


/*----------------------------------------------------------------------+
| Name:    QryMakeSeed()
|
| Action:  Creates the seed string as it was originally built when
|          stored.
|
| Input:
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
static int16 QryMakeSeed(seedV , seed_str)
   seedValidate  *seedV;
   char  *seed_str;
#else
static int16 QryMakeSeed(seedValidate *seedV,
                         char *seed_str)
#endif
{
char *cursor, *end_of_buff;
char seed_count[15] = "0";
int i,x,seed_len;

   seed_len = strlen(seed_str);
   cursor = seedV->seed;
   end_of_buff = seedV->seed + pref.sz_send_buff;
   while (end_of_buff > (cursor + seed_len + strlen(seed_count) + 1))
{
      memcpy(cursor,seed_str,seed_len);
      cursor += seed_len;
      x = sprintf(seed_count, "%d", ((cursor - seedV->seed) + 1));
      memcpy(cursor,seed_count,strlen(seed_count));
      cursor += strlen(seed_count);
   }
   for (i = end_of_buff - cursor; i ; --i )
   {
      *cursor = 0xFF ;
      cursor += 1 ;
   }

   return 0;
}

