/***********************************************************************
* Name:
*        xapilist.c
*
* Function:
*        Type and function definitions of the sorted object descriptor
*        list 
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        The file contains the functions that are invoked from the
*        query module (xapiqry.c). 
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xapiutil.h"       /* Handy utility routines.                 */
#include "custom.h"         /* Definitions for platform specific types */
#include "policy.h"         /* XOpen API type definitions              */
#include "xapint64.h"       /* 64 bit routines.                        */

#ifndef TRUE
#define TRUE  0x01
#endif
#ifndef FALSE
#define FALSE 0x00
#endif

typedef struct TElementStruct *TElement;
typedef struct TElementStruct {
                              ObjectDescriptor *actReference;
                              TElement         Descendant;
                              TElement         Predescessor;
                              } TElementDS;
               
typedef struct TListStruct {
                           TElement  Head;
                           TElement  End;
                           TElement  Current;
                           BSA_Int32 NumberOfObjects;
                           } TListDS;
typedef struct TListStruct *TList;

#define TypeList
#include "xapilist.h"       /* prototypes for the object descripor     */
                            /* list                                    */         
#undef  TypeList

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
TElement          Create_Element();
void              Dispose_Element();
ObjectDescriptor* ReadReference_Element();
void              WriteReference_Element();
void              Connect_Element();
void              Disconnect_Element();
TElement          Next_Element();
TElement          Back_Element();
#else /* use ANSI C required prototypes */
TElement          Create_Element();
void              Dispose_Element(TElement *actElement);
ObjectDescriptor* ReadReference_Element(TElement actElement);
void              WriteReference_Element(TElement          actElement,
                                         ObjectDescriptor* reference);
void              Connect_Element(TElement predescessor,
                                  TElement descendant);
void              Disconnect_Element(TElement element1,
                                     TElement element2);
TElement          Next_Element(TElement actElement);
TElement          Back_Element(TElement actElement);
#endif

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    Create_List()
|
| Action:  Creates and initializes a new sorted list to store object
|          descriptors.
|
| Input:   None
|
| Output:  None
|
| Returns: an empty list or NULL (no memory) 
|
| Side
| Effects: None
|
| Notes:   The returned list is empty.
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
TList Create_List()
#else
TList Create_List()
#endif

{

TList actList;

actList = (void*)malloc( sizeof( TListDS));
if (actList != NULL)
   {
   actList -> Head = Create_Element();
   actList -> End  = Create_Element();
   Connect_Element(actList->Head, actList->End);
   actList -> Current = actList -> End;
   actList -> NumberOfObjects = 0;  
   } /* if */

return actList;
} /* Create_List() */

/*----------------------------------------------------------------------+
| Name:    Dispose_List()
|
| Action:  Removes a list form the system and frees the memory.
|
| Input:   *actList - a pointer to an existing list
|
| Output:  *actList - a NULL pointer
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   If there are any object descriptors in the list they will 
|          be disposed too. The function to remove object descriptors
|          can be found in the xapiutil.c module. 
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void Dispose_List(actList)
   TList *actList;
#else
void Dispose_List(TList *actList)
#endif

{

GoStart_List(*actList);
while (!IsEmpty_List(*actList))
{
   Delete_List(*actList);
} /* while */
Dispose_Element(&((*actList)->Head));
Dispose_Element(&((*actList)->End));
free(*actList);
*actList = NULL;

return;
} /* Dispose_List() */

/*----------------------------------------------------------------------+
| Name:    Current_List()
|
| Action:  Returns the object descriptor at the current position of the
|          list.
|
| Input:   actList - an existing list
|
| Output:  None 
|
| Returns: pointer to an object descriptor
|
| Side
| Effects: None
|
| Notes:   It is assuemd that the current position of the list is
|          NOT OffStart and NOT OffEnd.
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
ObjectDescriptor* Current_List(actList)
   TList actList;
#else
ObjectDescriptor* Current_List(TList actList)
#endif

{

ObjectDescriptor *objectDescriptorPtr;

objectDescriptorPtr = ReadReference_Element(actList->Current);

return objectDescriptorPtr;
} /* Current_List() */

/*----------------------------------------------------------------------+
| Name:    Insert_List()
|
| Action:  Inserts a pointer to an object descriptor into the list. 
|
| Input:   actList              - an existing list
|          *objectDescriptorPtr - pointer to an object descriptor
|
| Output:  None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   The new element is inserted according to the value of the
|          restore order field. The list is sorted in ascending
|          order by the restoreOrder field.
|          Thereafter the new element will become the
|          current element of the list.
|          It is assumed that the current position of the list is
|          NOT OffStart and NOT OffEnd.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void Insert_List(actList, objectDescriptorPtr)
   TList actList;
   ObjectDescriptor *objectDescriptorPtr;
#else
void Insert_List(TList actList,
                 ObjectDescriptor *objectDescriptorPtr)
#endif

{

BSA_Boolean tmp;
TElement    newElement;
TElement    cut;

tmp = StopAt_List(actList, objectDescriptorPtr);
if (OffEnd_List(actList))
{
   Append_List(actList, objectDescriptorPtr);
   GoEnd_List(actList);
}
else
{ 
   newElement = Create_Element();
   WriteReference_Element(newElement, objectDescriptorPtr);
   if (IsEmpty_List(actList))
   {
      cut = actList->Head;
      actList->Current = actList->End;
   }
   else
   { 
      cut = Back_Element(actList->Current);
   } /* if */
   Disconnect_Element(cut, actList->Current);
   Connect_Element(cut, newElement);
   Connect_Element(newElement, actList->Current);
   actList->Current = newElement;
   (actList->NumberOfObjects)++;  
} /* if */

return;
} /* Insert_List() */

/*----------------------------------------------------------------------+
| Name:    Append_List()
|
| Action:  Appends an object descriptor pointer to the list.
|
| Input:   actList              - an existing list
|          *objectDescriptorPtr - pointer to an object descriptor
|
| Output:  None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   The current position of the list will not be changed.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void Append_List(actList, objectDescriptorPtr)
   TList actList;
   ObjectDescriptor *objectDescriptorPtr;
#else
void Append_List(TList actList,
                 ObjectDescriptor *objectDescriptorPtr)
#endif

{

TElement newElement;
TElement cut;

newElement = Create_Element();
WriteReference_Element(newElement, objectDescriptorPtr);
cut = Back_Element(actList->End);
Disconnect_Element(cut, actList->End);
Connect_Element(cut, newElement);
Connect_Element(newElement, actList->End);
(actList->NumberOfObjects)++;

return;
} /* Append_List() */

/*----------------------------------------------------------------------+
| Name:    Delete_List()
|
| Action:  Deletes the current element from the list. 
|          Note that not only the pointer to an object descriptor will
|          be deleted from the list but the whole object descriptor.
|
| Input:   actList              - an existing list
|
| Output:  None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   This function uses the dispose_objDesc() function contained
|          in the xapiutil module. 
|          It is assumed that the current position of the list is
|          NOT OffStart and NOT OffEnd.
|          After processing this function the element behind the deleted
|          one will become the current element. If you delete the last
|          object from a list the current position will be OffEnd.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void Delete_List(actList)
   TList actList;
#else
void Delete_List(TList actList)
#endif

{

TElement killMe;
TElement cut;
ObjectDescriptor *objectDescriptorPtr;

killMe = actList->Current;
cut = Back_Element(actList->Current);
actList->Current = Next_Element(actList->Current);
Disconnect_Element(cut ,killMe);
Disconnect_Element(killMe, actList->Current);
Connect_Element(cut, actList->Current);
objectDescriptorPtr = ReadReference_Element(killMe);
dispose_objDesc(objectDescriptorPtr);
Dispose_Element(&killMe);
(actList->NumberOfObjects)--;

return;
} /* Append_List() */

/*----------------------------------------------------------------------+
| Name:    StopAt_List()
|
| Action:  Searches for an object descriptor with the SameValue as the
|          passed object descriptor.
|
| Input:   actList              - an existing list
|          *objectDescriptorPtr - pointer to an object descriptor
|
| Output:  None
|
| Returns: TRUE                 - object descriptor with the same value
|                                 has been found. This object is the 
|                                 current element of the list.
|          FALSE                - no object descriptor with the same
|                                 value has been found
|
| Side
| Effects: None
|
| Notes:   None
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Boolean StopAt_List(actList, objectDescriptorPtr)
   TList actList;
   ObjectDescriptor *objectDescriptorPtr;
#else
BSA_Boolean StopAt_List(TList actList,
                        ObjectDescriptor *objectDescriptorPtr)
#endif

{

BSA_Boolean  found;
BSA_Boolean  goAhead;
ObjectDescriptor *actObjectPtr;

found = FALSE;
goAhead = !(IsEmpty_List(actList));
GoStart_List(actList);
while (goAhead)
{
   actObjectPtr = Current_List(actList);
   if (SmallerValue(actObjectPtr, objectDescriptorPtr))
   {
      StepNext_List(actList);
      goAhead = !(OffEnd_List(actList));
   }
   else
   {
      goAhead = FALSE;
      found = SameValue(actObjectPtr, objectDescriptorPtr);
   } /* if */
} /* while */

return found;
} /* StopAt_List */

/*----------------------------------------------------------------------+
| Name:    GoStart_List()
|
| Action:  The first object of the list becomes the current object. 
|
| Input:   actList - an existing list
|
| Output:  None 
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void GoStart_List(actList)
   TList actList;
#else
void GoStart_List(TList actList)
#endif

{

actList -> Current = Next_Element(actList->Head);

return;
} /* GoStart_List() */

/*----------------------------------------------------------------------+
| Name:    GoEnd_List()
|
| Action:  The last object of the list becomes the current object. 
|
| Input:   actList - an existing list
|
| Output:  None 
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void GoEnd_List(actList)
   TList actList;
#else
void GoEnd_List(TList actList)
#endif

{

actList -> Current = Back_Element(actList->End);

return;
} /* GoEnd_List() */

/*----------------------------------------------------------------------+
| Name:    StepNext_List()
|
| Action:  The next object of the list becomes the current object.
|
| Input:   actList - an existing list
|
| Output:  None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void StepNext_List(actList)
   TList actList;
#else
void StepNext_List(TList actList)
#endif

{

actList -> Current = Next_Element(actList->Current);

return;
} /* StepNext_List() */

/*----------------------------------------------------------------------+
| Name:    IsEmpty_List()
|
| Action:  Checks for an empty list. 
|
| Input:   actList - an existing list
|
| Output:  None
|
| Returns: TRUE    - list is empty
|          FALSE   - list contains at least one object descriptor 
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Boolean IsEmpty_List(actList)
   TList actList;
#else
BSA_Boolean IsEmpty_List(TList actList)
#endif

{

BSA_Boolean rc;

if (actList->NumberOfObjects == 0)
{
   rc = TRUE;
}
else
{
   rc = FALSE;
} /* if */

return rc;
} /* IsEmpty_List() */

/*----------------------------------------------------------------------+
| Name:    OffStart_List()
|
| Action:  The function returns TRUE if the current position is in front
|          of the first object of the list. 
|
| Input:   actList - an existing list
|
| Output:  None
|
| Returns: TRUE    - current position is in front of the first object of
|                    the list
|          FALSE   - current position is NOT in front of the first
|                    object of the list 
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Boolean OffStart_List(actList)
   TList actList;
#else
BSA_Boolean OffStart_List(TList actList)
#endif

{

BSA_Boolean rc;

if (actList->Head == actList->Current)
{
   rc = TRUE;
}
else
{
   rc = FALSE;
} /* if */

return rc;
} /* OffStart_List() */

/*----------------------------------------------------------------------+
| Name:    OffEnd_List()
|
| Action:  The function returns TRUE if the current position is behind 
|          the last object of the list.
|
| Input:   actList - an existing list
|
| Output:  None
|
| Returns: TRUE    - current position is behind the last object of
|                    the list
|          FALSE   - current position is NOT behind the last 
|                    object of the list
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Boolean OffEnd_List(actList)
   TList actList;
#else
BSA_Boolean OffEnd_List(TList actList)
#endif

{

BSA_Boolean rc;

if (actList->End == actList->Current)
{
   rc = TRUE;
}
else
{
   rc = FALSE;
} /* if */

return rc;
} /* OffEnd_List() */

/*----------------------------------------------------------------------+
| Private routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    Create_Element() 
|
| Action:  Creates and initializes a new element of a list. 
|
| Input:   None
|
| Output:  None
|
| Returns: a new element or NULL (no memory)
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
TElement Create_Element()
#else
TElement Create_Element()
#endif

{

TElement actElement;

actElement = (void*)malloc( sizeof( TElementDS));
if (actElement != NULL)
   {
   actElement -> Descendant   = NULL;
   actElement -> Predescessor = NULL;
   }

return actElement;
} /* Create_Element() */

/*----------------------------------------------------------------------+
| Name:    Dispose_Element() 
|
| Action:  Removes an element form the system. 
|
| Input:   *actElement - a pointer to an existing element
|
| Output:  *actElement - a NULL pointer 
|
| Returns: None 
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void Dispose_Element(actElement)
   TElement *actElement;
#else
void Dispose_Element(TElement *actElement)
#endif

{

free(*actElement);
*actElement = NULL;

return;
} /* Dispose_Element() */

/*----------------------------------------------------------------------+
| Name:    ReadReference_Element()
|
| Action:  Reads the reference form an element. 
|
| Input:   actElement - an existing element
|
| Output:  None 
|
| Returns: object descriptor
|
| Side
| Effects: None
|
| Notes:   Every element contains one pointer to an object descriptor.
|          This function reads this pointer (reference of an object
|          descriptor) and returns it.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
ObjectDescriptor* ReadReference_Element(actElement)
   TElement actElement;
#else
ObjectDescriptor* ReadReference_Element(TElement actElement)
#endif

{

ObjectDescriptor *actRef;

actRef = actElement -> actReference;

return actRef;
} /* ReadReference_Element() */

/*----------------------------------------------------------------------+
| Name:    WriteReference_Element()
|
| Action:  Writes the reference to an element.
|
| Input:   actElement - an existing element
|          reference  - a pointer to an object descriptor (reference)
|
| Output:  None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   Every element contains one pointer to an object descriptor.
|          This function writes this pointer (reference of an object
|          descriptor).
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void WriteReference_Element(actElement, reference)
   TElement          actElement;
   ObjectDescriptor* reference;
#else
void WriteReference_Element(TElement          actElement,
                            ObjectDescriptor* reference)
#endif

{

actElement -> actReference = reference;

return;
} /* WriteReference_Element() */

/*----------------------------------------------------------------------+
| Name:    Connect_Element()
|
| Action:  Connects two elements. 
|
| Input:   predescessor - an existing element
|          descendant   - an existing element
|
| Output:  predescessor - element predescessor is connected to
|                         descendant 
|          descendant   - element descendant is connected to
|                         predescessor
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   Every element contains two pointers to elements. This 
|          pointers are used to point to a descendant and a
|          predescessor of the current element. 
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void Connect_Element(predescessor, descendant)
   TElement predescessor;
   TElement descendant;
#else
void Connect_Element(TElement predescessor,
                     TElement descendant)
#endif

{

predescessor -> Descendant   = descendant;
descendant   -> Predescessor = predescessor;
 
return;
} /* Connect_Element() */

/*----------------------------------------------------------------------+
| Name:    Disconnect_Element()
|
| Action:  Disconnects two elements.
|
| Input:   element1 - an existing element
|          element2 - an existing element
|
| Output:  element1 - element1 is not connected to element2
|          element2 - element2 is not connected to element1
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   Every element contains two pointers to elements. The
|          passed elements will be disconnected only if they were
|          previously connected.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void Disconnect_Element(element1, element2)
   TElement element1;
   TElement element2;
#else
void Disconnect_Element(TElement element1,
                        TElement element2)
#endif

{

if (element1->Descendant == element2)
{
   element1->Descendant   = NULL;
   element2->Predescessor = NULL;
}
else
{
   if (element2->Descendant == element1)
   {
      element2->Descendant   = NULL;
      element1->Predescessor = NULL;
   }
   else
   {
   /* element 1 and element 2 are not connected. */
   } /* if */
} /* if */

return;
} /* Disconnect_Element() */

/*----------------------------------------------------------------------+
| Name:    Next_Element()
|
| Action:  Returns the descendant of the current element. 
|
| Input:   actElement - an existing element
|
| Output:  None
|
| Returns: element
|
| Side
| Effects: None
|
| Notes:   
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
TElement Next_Element(actElement)
   TElement actElement;
#else
TElement Next_Element(TElement actElement)
#endif

{

return actElement->Descendant;
} /* Next_Element() */

/*----------------------------------------------------------------------+
| Name:    Back_Element()
|
| Action:  Returns the predescessor of the current element.
|
| Input:   actElement - an existing element
|
| Output:  None
|
| Returns: element
|
| Side
| Effects: None
|
| Notes:   
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
TElement Back_Element(actElement)
   TElement actElement;
#else
TElement Back_Element(TElement actElement)
#endif

{

return actElement->Predescessor;
} /* Back_Element() */

/*----------------------------------------------------------------------+
| Name:    SmallerValue()
|
| Action:  Compares the restoreOrder field of the object descriptors.
|
| Input:   *objectDescPtr1 - pointer to an object descriptor
|          *objectDescPtr2 - pointer to an object descriptor
|
| Output:  None
|
| Returns: TRUE                 - restoreOrder field of objectDescPtr1
|                                 has a smaller value than
|                                 objectDescPtr2
|          FALSE                - restoreOrder field of objectDescPtr1
|                                 has NO smaller value than
|                                 objectDescPtr2
|
| Side
| Effects: None
|
| Notes:   None
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Boolean SmallerValue(objectDescPtr1, objectDescPtr2)
   ObjectDescriptor *objectDescPtr1;
   ObjectDescriptor *objectDescPtr2;
#else
BSA_Boolean SmallerValue(ObjectDescriptor *objectDescPtr1,
                         ObjectDescriptor *objectDescPtr2)
#endif

{

BSA_Boolean smaller;

smaller = FALSE;
if (Cmp64(objectDescPtr1->restoreOrder,
          objectDescPtr2->restoreOrder) == LESSTHAN)
{
   smaller = TRUE;
}
else
{
   smaller = FALSE;
} /* if */

return smaller; 
} /* SmallerValue */

/*----------------------------------------------------------------------+
| Name:    SameValue()
|
| Action:  Compares the restoreOrder field of the object descriptors.
|
| Input:   *objectDescPtr1 - pointer to an object descriptor
|          *objectDescPtr2 - pointer to an object descriptor
|
| Output:  None
|
| Returns: TRUE                 - restoreOrder field of objectDescPtr1
|                                 has the same value as 
|                                 objectDescPtr2
|          FALSE                - restoreOrder field of objectDescPtr1
|                                 has NOT the same value as
|                                 objectDescPtr2
|
| Side
| Effects: None
|
| Notes:   None
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Boolean SameValue(objectDescPtr1, objectDescPtr2)
   ObjectDescriptor *objectDescPtr1;
   ObjectDescriptor *objectDescPtr2;
#else
BSA_Boolean SameValue(ObjectDescriptor *objectDescPtr1,
                      ObjectDescriptor *objectDescPtr2)
#endif

{

BSA_Boolean same;

same = FALSE;
if (Cmp64(objectDescPtr1->restoreOrder,
          objectDescPtr2->restoreOrder) == EQUAL)
{
   same = TRUE;
}
else
{
   same = FALSE;
} /* if */

return same; 
} /* SameValue */

