/***********************************************************************
* Name:
*        xapiour.c
*
* Function:
*        Functions of our API definition.
*        Transforms return codes into text messages.
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Description:
*        The file contains the functions that are invoked from the
*        Session, Send, Receive and Query modules.
***********************************************************************/
#include <stdlib.h>

#include "xapiour.h"        /* our API functions                       */
#include "xapidef.h"        /* XOpen Sample API type definitions       */

static BSA_UInt32 BSAHandle = 0;

/*----------------------------------------------------------------------+
| Public routines
.----------------------------------------------------------------------*/

/*----------------------------------------------------------------------+
| Name:    ourQueryApiVersion()
|
| Action:  Returns the API version of the currently used library.
|
| Input:   None
|
| Output:  pointer to the API version structure
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void ourQueryApiVersion(apiVersion)
   ApiVersion *apiVersion;
#else /* use ANSI C required prototypes */
void ourQueryApiVersion(ApiVersion *apiVersion)
#endif

{

memset(apiVersion,0x00,sizeof(ApiVersion));
BSAQueryApiVersion(apiVersion);
 
return;
} /* ourQueryApiVersion() */

/*----------------------------------------------------------------------+
| Name:    ourResolveLifecycleGroup()
|
| Action:  The BSAResolveLifecycleGroup() call determines the lifecycle
|          group to be associated with the object.
|
| Input:   objectDescriptorPtr - X/Open API data structure describing
|                                an object
|
| Returns: RC_OK               - Successful
|          others              - Any other valid X/Open api rc.
|          objectDescriptorPtr - This parameter points to a structure
|                                that contains the attributes of the
|                                object.
|
| Side
| Effects: None
|
| Notes:   This function assumes that you are already signed on.
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourResolveLifecycleGroup()
   ObjectDescriptor *objectDescriptorPrt;
#else /* use ANSI C required prototypes */
BSA_Int16 ourResolveLifecycleGroup(
                  ObjectDescriptor *objectDescriptorPtr )
#endif

{

BSA_Int16   rc;
BSA_Boolean verbose;

verbose = get_verbose();
rc = BSAResolveLifecycleGroup( BSAHandle, /* BSA session Handle */
                               objectDescriptorPtr );
if  (rc != BSA_RC_OK)
{
   printf("*** BSAResolveLifecycleGroup failed: ");
   rcXApiOut(rc);
   rc = (RC_SESSION_FAILED);
   return rc;
} /* if */

/*----------------------------------------------------------------------+
| Check if a copy group is defined for the object in that
| lifecycle group.
.----------------------------------------------------------------------*/
if (objectDescriptorPtr->cGName[0] == 0x00)       /* empty string?     */
{
   printf("*** Object %s %s will not be sent because\n",
          objectDescriptorPtr->objName.objectSpaceName,
          objectDescriptorPtr->objName.pathName);
   printf("    the lifecycle group: %s\n",
          objectDescriptorPtr->lGName);
   printf("    does not have a ");
   switch(objectDescriptorPtr->copyType)
   {
      case(BSACopyType_ANY):
         printf("'ANY'");
         break;
      case(BSACopyType_ARCHIVE):
         printf("archive");
         break;
      case(BSACopyType_BACKUP):
      default:
         printf("backup");
   } /* switch */
   printf(" copy group defined.\n");
   rc = (RC_SESSION_FAILED);
   return rc;
} /* if */

if (verbose)
{
   printf("Lifecycle Group name: %s\n",objectDescriptorPtr->lGName);
   printf("     Copy Group name: %s\n",objectDescriptorPtr->cGName);
} /* if */

return RC_OK;
} /* ourResolveLifecycleGroup() */

/*----------------------------------------------------------------------+
| Name:   ourCreateObject()
|
| Action: Creates an object
|
| Input:  SendBuffer          - Buffer with data to send
|         objectDescriptorPtr - This parameter points to a structure
|                               that contains the attributes of the
|                               object.
|
|
| Returns: RC_OK              - Successful
|          RC_NO_MEMORY       - Not enough memory to create a data block
|          others             - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourCreateObject()
   char             *SendBuffer;
   ObjectDescriptor *objectDescriptorPtr;
#else /* use ANSI C required prototypes */
BSA_Int16 ourCreateObject( char *SendBuffer,
               ObjectDescriptor *objectDescriptorPtr )
#endif

{

BSA_Int16     rc;
DataBlock     *dataBlockPtr;
BSA_UInt32    SendBufferSize;

SendBufferSize = get_send_buff_size();
/*----------------------------------------------------------------------
| Create a Data Block and initialize the pointers.
.----------------------------------------------------------------------*/
dataBlockPtr = (void *)malloc( sizeof( DataBlock));
if (dataBlockPtr == NULL)
   {
   /* no memory available */
   rc = RC_NO_MEMORY;
   return rc;
   } /* if */
dataBlockPtr->bufferLen = SendBufferSize;
dataBlockPtr->numBytes  = 0;
dataBlockPtr->bufferPtr = SendBuffer;

rc = BSACreateObject( BSAHandle,       /* BSA session Handle */
                      objectDescriptorPtr,
                      dataBlockPtr );
if (rc)
{
   printf("*** BSACreateObject failed: ");
   rcXApiOut(rc);
   rc = (RC_SESSION_FAILED);
   return rc;
} /* if */

return RC_OK;
} /* ourCreateObject() */

/*----------------------------------------------------------------------+
| Name:   ourSendData()
|
| Action: Sends the data of the buffer to the server
|
| Input:  SendBuffer          - Buffer with data to send
|         BufferLength        - Length of the buffer
|
|
| Returns: RC_OK              - Successful
|          RC_NO_MEMORY       - Not enough memory to create a data block
|          others             - Any other valid X/Open api rc.
|
| Side
| Effects: None
|
| Notes:   With the BufferLength parameter you can specify the
|          actual length of the buffer. This is needed if only a part
|          of an existing buffer should be send.
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourSendData()
   char       *SendBuffer;
   BSA_UInt16 BufferLength
#else /* use ANSI C required prototypes */
BSA_Int16 ourSendData( char *SendBuffer,
                 BSA_UInt32 BufferLength )
#endif

{

BSA_Int16     rc;
DataBlock     *dataBlockPtr;

/*----------------------------------------------------------------------
| Create a Data Block and initialize the pointers.
.----------------------------------------------------------------------*/
dataBlockPtr = (void *)malloc( sizeof( DataBlock));
if (dataBlockPtr == NULL)
   {
   /* no memory available */
   rc = RC_NO_MEMORY;
   return rc;
   } /* if */
dataBlockPtr->bufferLen = BufferLength;
dataBlockPtr->numBytes  = 0;
dataBlockPtr->bufferPtr = SendBuffer;

rc = BSASendData( BSAHandle,      /* BSA session Handle */
                  dataBlockPtr );
if (rc)
{
   printf("*** BSASendData failed: ");
   rcXApiOut(rc);
   rc = (RC_SESSION_FAILED);
   return rc;
} /* if */

return RC_OK;
} /* ourSendData() */

/*----------------------------------------------------------------------+
| Name:   ourInit()
|
| Action: Initializes the environment and sets up a session with
|         a server.
|
| Input:  *tokenPtr           - a pointer to the security token
|         *objectOwnerPtr     - pointer to the object owner structure
|
| Returns: RC_OK             - Successful
|          RC_SESSION_FAILED - BSAInit returned an error code 
|
| Side
| Effects: None
|
| Notes:   Prints the environment structure.
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourInit()
   SecurityToken *tokenPtr;
   ObjectOwner   *objectOwnerPtr;
#else /* use ANSI C required prototypes */
BSA_Int16 ourInit(SecurityToken *tokenPtr,
                  ObjectOwner   *objectOwnerPtr)
#endif

{

char      *envPtr[ADSM_ENV_STRS+1];
char      envStrs[ADSM_ENV_STRS] [BSA_MAX_DESC];
BSA_Int16 rc = RC_OK;
int i; 

for (i=0; i<ADSM_ENV_STRS; i++)
   envPtr[i] = envStrs[i];
envPtr[ADSM_ENV_STRS] = NULL;

rc = BSAInit(&BSAHandle,      /* Will contain session handle on retrun. */
             tokenPtr,        /* password                               */
             objectOwnerPtr,  /* Node name and Owner name               */
             envPtr);         /* Environment variables                  */

if (rc)
{
   printf("*** BSAInit failed: ");
   rcXApiOut(rc);
   ourTerminate();       /* clean up memory blocks */
   rc = (RC_SESSION_FAILED);
   return rc;
}
else
{
   printf("BSAHandle on return = %lu\n",BSAHandle);

   rc = ourGetEnvironment(objectOwnerPtr, envPtr);
   printf("ADSM node/bsaObjectOwner = %s\n",objectOwnerPtr->bsaObjectOwner);
   printf("ADSM owner/appObjectOwner = %s\n",objectOwnerPtr->appObjectOwner);
   printf("BSA Environment:\n");
   for (i=0; i<ADSM_ENV_STRS; i++)
      printf(" %s\n", envPtr[i]);

   printf("\n");
} /* if */

return rc;
} /* ourInit() */

/*----------------------------------------------------------------------+
| Name:    ourBeginTxn()
|
| Action:  Starts the transaction with the server.
|
| Input:
|
| Returns: RC_OK             - Successful
|          RC_SESSION_FAILED - BSABeginTxn returned an error code
|
| Side
| Effects: None
|
| Notes:   This function assumes that you are already signed on.
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourBeginTxn()
#else /* use ANSI C required prototypes */
BSA_Int16 ourBeginTxn()
#endif

{

BSA_Int16 rc = RC_OK;

/*----------------------------------------------------------------------+
| Start the transaction
.----------------------------------------------------------------------*/
rc = BSABeginTxn( BSAHandle );         /* BSA session Handle */
if (rc)
{
   printf("*** BSABeginTxn failed: ");
   rcXApiOut(rc);
   rc = (RC_SESSION_FAILED);
} /* if */

return rc;
} /* ourBeginTxn() */

/*----------------------------------------------------------------------+
| Name:    ourEndTxn()
|
| Action:  Ends the transaction with the server.
|
| Input:   vote              - BSA_VOTE_COMMIT or BSA_VOTE_ABORT
|
| Returns: RC_OK             - Successful
|          RC_SESSION_FAILED - BSAEndTxn returned an error code
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourEndTxn()
   Vote vote;
#else /* use ANSI C required prototypes */
BSA_Int16 ourEndTxn(Vote vote)
#endif

{

BSA_Int16 rc = RC_OK;

/*----------------------------------------------------------------------+
| Ends the transaction
.----------------------------------------------------------------------*/
rc = BSAEndTxn( BSAHandle, vote );          /* BSA session Handle */
if (rc)
{
   printf("*** BSAEndTxn failed: ");
   rcXApiOut(rc);
   rc = (RC_SESSION_FAILED);
} /* if */

return rc;
} /* ourEndTxn() */

/*----------------------------------------------------------------------+
| Name:    ourEndData()
|
| Action:  Ends a BSASendData() sequence.
|
| Input:
|
| Returns: RC_OK             - Successful
|          RC_SESSION_FAILED - BSAEndData returned an error code
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourEndData()
#else /* use ANSI C required prototypes */
BSA_Int16 ourEndData()
#endif

{

BSA_Int16 rc = RC_OK;

/*----------------------------------------------------------------------+
| Ends a BSASendData() sequence
.----------------------------------------------------------------------*/
rc = BSAEndData( BSAHandle );         /* BSA session Handle */
if (rc)
{
   printf("*** BSAEndData failed: ");
   rcXApiOut(rc);
   rc = (RC_SESSION_FAILED);
} /* if */

return rc;
} /* ourEndData() */

/*----------------------------------------------------------------------+
| Name:    ourGetObject()
|
| Action:  Receives an object form the server.
|
| Input:   *objectDescriptorPtr - object descriptor for the object to
|                                 receive
|          *dataBlockPtr        - pointer to the data block
|
| Returns: BSA_RC_OK            - Successful
|          BSA_RC_MORE_DATA     - the data in the buffer is valid
|          BSA_RC_NO_MORE_DATA  - the data in the buffer is NOT part
|                                 of the object
|          RC_SESSION_FAILED    - BSAGetObject returned an error code
|
| Side
| Effects: None
|
| Notes:   This function assumes that you are already signed on.
|          Note that the BSAGetObject call returns BSA_RC_MORE_DATA
|          in case the returned buffer contains valid data.
|          If the return code is BSA_RC_NO_MORE_DATA the returned
|          buffer does not contain any useful information.
|
|          Implementation example:
|          rc = BSAGetObject( BSAHandle,
|                             objectDescriptorPtr,
|                             dataBlockPtr);
|          while (rc == BSA_RC_MORE_DATA)
|          {
|             // use data from the buffer
|             rc = BSAGetData( BSAHandle,
|                              dataBlockPtr );
|          }  // end while
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourGetObject()
   ObjectDescriptor *objectDescriptorPtr;
   DataBlock        *dataBlockPtr;
#else /* use ANSI C required prototypes */
BSA_Int16 ourGetObject(ObjectDescriptor *objectDescriptorPtr,
                       DataBlock        *dataBlockPtr)
#endif

{

BSA_Int16 rc = BSA_RC_OK;

rc = BSAGetObject( BSAHandle,                  /* BSA session Handle */
                   objectDescriptorPtr,
                   dataBlockPtr);
if (((rc != BSA_RC_OK) &&
     (rc != BSA_RC_MORE_DATA)) &&
     (rc != BSA_RC_NO_MORE_DATA))
{
   printf("*** BSAGetObject failed: ");
   rcXApiOut(rc);
   rc = (RC_SESSION_FAILED);
} /* if */

return rc;
} /* ourGetObject() */

/*----------------------------------------------------------------------+
| Name:    ourQueryObject()
|
| Action:  Initiates a request for information on object copies from
|          Backup Services.
|
| Input:   *queryDescriptorPtr  - pointer to a query descriptor
|
| Output:  *objectDescriptorPtr - pointer to an object descriptor
|
| Returns: BSA_RC_OK            - Successful
|          BSA_RC_MORE_DATA     - the data in the buffer is valid
|          BSA_RC_NO_MATCH      - no object on the server matches the
|                                 query condition
|          BSA_RC_NO_MORE_DATA  - the data in the buffer is NOT part
|                                 of the object
|          RC_SESSION_FAILED    - BSAQueryObject returned an error code 
|
| Side
| Effects: None
|
| Notes:   This function assumes that you are already signed on.
|          Note that the BSAQueryObject call returns BSA_RC_MORE_DATA
|          in case the returned ObjectDescriptor structure is valid.
|          If the return code is BSA_RC_NO_MORE_DATA the returned
|          object descriptor structure does not contain an existing
|          object descriptor.
|
|          Implementation example:
|          rc = BSA_RC_MORE_DATA;
|          while (rc == BSA_RC_MORE_DATA)
|          {
|             rc = BSAQueryObject( BSAHandle,
|                                  queryDescriptorPtr,
|                                  objectDescriptorPtr );
|             // insert object descriptor into list
|          }  // end while
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourQueryObject(queryDescriptorPtr, objectDescriptorPtr)
   QueryDescriptor  *queryDescriptorPtr;
   ObjectDescriptor *objectDescriptorPtr;
#else /* use ANSI C required prototypes */
BSA_Int16 ourQueryObject(QueryDescriptor  *queryDescriptorPtr,
                         ObjectDescriptor *objectDescriptorPtr)
#endif

{

BSA_Int16 rc;

rc = BSAQueryObject( BSAHandle,           /* BSA session Handle */
                     queryDescriptorPtr,
                     objectDescriptorPtr );
if (((rc != BSA_RC_OK) &&
    ((rc != BSA_RC_MORE_DATA) &&
     (rc != BSA_RC_NO_MATCH))) &&
     (rc != BSA_RC_NO_MORE_DATA))
{
   printf("*** BSAQueryObject failed: ");
   rcXApiOut(rc);
   rc = RC_SESSION_FAILED;
} /* if */

return rc;
} /* ourQueryObject() */

/*----------------------------------------------------------------------+
| Name:    ourGetNextQueryObject()
|
| Action:  Returns the next object descriptor in response to a
|          previous query.
|
| Input:   None
|
| Output:  *objectDescriptorPtr - pointer to an object descriptor
|
| Returns: BSA_RC_OK            - Successful
|          BSA_RC_MORE_DATA     - the data in the buffer is valid
|          BSA_RC_NO_MORE_DATA  - the data in the buffer is NOT part
|                                 of the object
|          RC_SESSION_FAILED    - BSAGetNextQueryObject returned an
|                                 error code
|
| Side
| Effects: None
|
| Notes:   This function assumes that you are already signed on.
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourGetNextQueryObject(objectDescriptorPtr)
   ObjectDescriptor *objectDescriptorPtr;
#else /* use ANSI C required prototypes */
BSA_Int16 ourGetNextQueryObject(ObjectDescriptor  *objectDescriptorPtr)
#endif

{

BSA_Int16 rc = BSA_RC_OK;

rc = BSAGetNextQueryObject( BSAHandle,     /* BSA session Handle */
                            objectDescriptorPtr );
if (((rc != BSA_RC_OK) &&
     (rc != BSA_RC_MORE_DATA)) &&
     (rc != BSA_RC_NO_MORE_DATA))
{
   printf("*** BSAGetNextQueryObject failed: ");
   rcXApiOut(rc);
   rc = RC_SESSION_FAILED;
} /* if */

return rc;
} /* ourGetNextQueryObject() */

/*----------------------------------------------------------------------+
| Name:    ourGetData()
|
| Action:  Receives data from Backup Services.
|          previous query.
|
| Input:   *dataBlockPtr - pointer to an initialized data block for
|                          the data that is to be received
|
| Output:  *dataBlockPtr - pointer to a data block that contains the
|                          received data
|
| Returns: BSA_RC_OK            - Successful
|          BSA_RC_MORE_DATA     - the data in the buffer is valid
|          BSA_RC_NO_MORE_DATA  - the data in the buffer is NOT part
|                                 of the object
|          RC_SESSION_FAILED    - BSAGetNextQueryObject returned an
|                                 error code
|
| Side
| Effects: None
|
| Notes:   This function assumes that you are already signed on.
|          Note that the BSAGetData call returns BSA_RC_MORE_DATA
|          in case the returned buffer contains valid data.
|          If the return code is BSA_RC_NO_MORE_DATA the returned
|          buffer does not contain any useful information.
|
|          Implementation example:
|          rc = BSAGetObject( BSAHandle,
|                             objectDescriptorPtr,
|                             dataBlockPtr);
|          while (rc == BSA_RC_MORE_DATA)
|          {
|             // use data from the buffer
|             rc = BSAGetData( BSAHandle,
|                              dataBlockPtr );
|          }  // end while
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourGetData(dataBlockPtr)
   DataBlock *dataBlockPtr;
#else /* use ANSI C required prototypes */
BSA_Int16 ourGetData(DataBlock *dataBlockPtr)
#endif

{

BSA_Int16 rc;

rc = BSAGetData( BSAHandle,     /* BSA session Handle */
                 dataBlockPtr );
if (((rc != BSA_RC_OK) &&
     (rc != BSA_RC_MORE_DATA)) &&
     (rc != BSA_RC_NO_MORE_DATA))
{
   printf("*** BSAGetData failed: ");
   rcXApiOut(rc);
   rc = (RC_SESSION_FAILED);
} /* if */

return RC_OK;
} /* ourGetData() */

/*----------------------------------------------------------------------+
| Name:    ourTerminate()
|
| Action:  Terminates a session with Backup Services 
|
| Input:   None
|
| Output:  None
|
| Returns: BSA_RC_OK            - Successful
|          other                - BSATerminate returned an error code 
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourTerminate()
#else /* use ANSI C required prototypes */
BSA_Int16 ourTerminate()
#endif

{

BSA_Int16 rc = BSA_RC_OK;

rc = BSATerminate(BSAHandle);     /* BSA session Handle */
BSAHandle = 0;

if (rc != BSA_RC_OK)
{
   printf("*** BSATerminate failed: ");
   rcXApiOut(rc);
} /* if */

return rc;
} /* ourTerminate() */

/*----------------------------------------------------------------------+
| Name:    ourChangeToken()
|
| Action:  Changes the signon password.
|
| Input:   *oldTokenPtr - pointer to the old security token
|          *newTokenPtr - pointer to the new security token
|
| Output:  None
|
| Returns: BSA_RC_OK            - Successful
|          other                - BSAChangeToken returned an error code 
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourChangeToken(oldTokeniPtr, newTokenPtr)
   SecurityToken *oldTokenPtr;
   SecurityToken *newTokenPtr;
#else /* use ANSI C required prototypes */
BSA_Int16 ourChangeToken(SecurityToken *oldTokenPtr,
                         SecurityToken *newTokenPtr)
#endif

{

BSA_Int16 rc = BSA_RC_OK;

rc = BSAChangeToken(BSAHandle, oldTokenPtr, newTokenPtr);
if (rc != BSA_RC_OK)
{
   printf("*** BSAChangeToken failed: ");
   rcXApiOut(rc);
}
else
{
   printf("*** Your new password has been accepted and updated.\n");
} /* if */
 
return rc;
} /* ourChangeToken() */

/*----------------------------------------------------------------------+
| Name:    check_signon() 
|
| Action:  Checks if the application is already signed on to the server 
|
| Input:
|
| Output:  RC_OK             - application is signed on
|          RC_NOT_IN_SESSION - not signed on to the server
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 check_signon()
#else
BSA_Int16 check_signon()
#endif

{

BSA_Int16 rc = RC_OK;

if (!BSAHandle)
{
   printf("You must signon before performing this action.\n");
   rc = (RC_NOT_IN_SESSION);
} /* if */

return rc;

} /* check_signon() */

/*----------------------------------------------------------------------+
| Name:    ourGetEnvironment()
|
| Action:  Retrieves the current environment for the session.
|
| Input:   *environmentPtr - pointer to a structure which returns the
|                            environment information
|
| Output:  *environmentPtr - pointer to a structure with environment
|                            information
|          *objectOwnerPtr - pointer to a structure with the BSA User
|                            name and the Application User Name
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourGetEnvironment(objectOwnerPtr, environmentPtr)
   ObjectOwner    *objectOwnerPtr;
   char          **environmentPtr;
#else /* use ANSI C required prototypes */
BSA_Int16 ourGetEnvironment(ObjectOwner *objectOwnerPtr,
                            char        **environmentPtr)
#endif

{

BSA_Int16 rc = RC_OK;

rc = BSAGetEnvironment(BSAHandle, 
                       objectOwnerPtr,
                       environmentPtr);
 
return rc;
} /* ourGetEnvironment() */

/*----------------------------------------------------------------------+
| Name:    ourDeleteObject()
|
| Action:  Deletes a BSA object. (Only archive objects in ADSM)
|
| Input:   copyType    - specifies the type of the copy
|                        (ADSM supports only BSACopyType_ARCHIVE)
|          *objNamePtr - pointer to the name of the object to be
|                        deleted
|          *copyIdPtr  - pointer to the id of the object
|
| Output:  None
|
| Returns: BSA_RC_OK            - Successful
|          other                - BSADeleteObject returned an error code 
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourDeleteObject()
   CopyType   copyType;
   ObjectName *objNamePtr;
   CopyId     *copyIdPtr;
#else /* use ANSI C required prototypes */
BSA_Int16 ourDeleteObject(CopyType   copyType,
                          ObjectName *objNamePtr,
                          CopyId     *copyIdPtr)
#endif

{

BSA_Int16 rc = BSA_RC_OK;

rc = BSADeleteObject(BSAHandle,      /* BSA session Handle */
                     copyType,
                     objNamePtr,
                     copyIdPtr); 

if (rc != BSA_RC_OK)
{
   printf("*** BSADeleteObject failed: ");
   rcXApiOut(rc);
} /* if */

return rc;
} /* ourDeleteObject() */

/*----------------------------------------------------------------------+
| Name:    ourDeleteObject()
|
| Action:  Deletes a BSA object. (Only archive objects in ADSM)
|
| Input:   None
|
| Output:  None
|
| Returns: BSA_RC_OK            - Successful
|          other                - BSADeleteObject returned an error code 
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 ourMarkObjectInactive()
   ObjectName *objectNamePtr;
#else /* use ANSI C required prototypes */
BSA_Int16 ourMarkObjectInactive(ObjectName *objectNamePtr)
#endif

{

BSA_Int16 rc = BSA_RC_OK;

rc = BSAMarkObjectInactive(BSAHandle,      /* BSA session Handle */
                           objectNamePtr); 

if (rc != BSA_RC_OK)
{
   printf("*** BSAMarkObjectInactive failed: ");
   rcXApiOut(rc);
} /* if */

return rc;
} /* ourMarkObjectInactive() */

/*----------------------------------------------------------------------+
| Name:   rcXApiOut()
|
| Action: Transforms the return codes into text messages.
|
| Input:   rc                - return code
|
| Output:  None
|
| Returns: None
|
| Side
| Effects: Prints an error message to standard output.
|
| Notes:   None
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void rcXApiOut(rc)
   BSA_Int16 rc;
#else /* use ANSI C required prototypes */
void rcXApiOut(BSA_Int16 rc)
#endif
{
   printf("(%d) ",rc);
   switch(rc)
   {
      case BSA_RC_OK :
         printf("No error occured.\n");
         break;
      case BSA_RC_ABORT_ACTIVE_NOT_FOUND :
         printf("No active object matched the name that was\n");
         printf("specified for a BSADeactivateObject() call.\n");
         break;
      case BSA_RC_ABORT_SYSTEM_ERROR :
         printf("System detected error, operation aborted.\n");
         break;
      case BSA_RC_AUTHENTICATION_FAILURE :
         printf("There was an authentication failure.\n");
         break;
      case BSA_RC_BAD_CALL_SEQUENCE :
         printf("The sequence of API calls is incorrect.\n");
         break;
      case BSA_RC_BAD_HANDLE :
         printf("The handle used to associate this call with a\n");
         printf("previous BSAInit() call is invalid.\n");
         break;
      case BSA_RC_BUFFER_TOO_SMALL :
         printf("Buffer is too small to hold the object entry to\n");
         printf("be returned.\n");
         break;
      case BSA_RC_DESC_TOO_LONG :
         printf("The description is too long.\n");
         break;
      case BSA_RC_OBJECTSPACE_TOO_LONG :
         printf("The length of the objectspace name is too long.\n");
         break;
      case BSA_RC_INVALID_TOKEN :
         printf("The new token is invalid.\n");
         break;
      case BSA_RC_INVALID_VOTE :
         printf("The value specified for vote is invalid.\n");
         break;
      case BSA_RC_INVALID_KEYWORD :
         printf("An entry in the environment structure is\n");
         printf("invalid.\n");
         break; 
      case BSA_RC_MATCH_EXISTS :
         printf("Object matching the specified predicate\n");
         printf("already exists.\n");
         break;
      case BSA_RC_MORE_DATA :
         printf("More data is available. Data can be obtained\n");
         printf("through BSAGetData() or ");
         printf("BSAGetNextQueryObject().\n");
         break;
      case BSA_RC_MORE_RULES :
         printf("Response has more rules. Remainder of the\n");
         printf("rules can obtained through successive ");
         printf("BSAGetNextAccessRule() calls.\n");
         break;
      case BSA_RC_NEWTOKEN_REQD :
         printf("A value must be entered for the new token.\n");
         printf("The old token has expired.\n");
         break;
      case BSA_RC_NO_MATCH :
         printf("No object matched the specified predicate.\n");
         break;
      case BSA_RC_NO_MORE_DATA :
         printf("There is no more data.\n");
         break;
      case BSA_RC_NO_RESOURCES :
         printf("Backup_Services is out of resources.\n");
         break;
      case BSA_RC_NULL_DATABLKPTR :
         printf("A data block pointer is required.\n");
         break;
      case BSA_RC_NULL_OBJNAME :
         printf("An object name is required.\n");
         break;
      case BSA_RC_NULL_POINTER :
         printf("A required pointer parameter is NULL.\n");
         break;
      case BSA_RC_NULL_RULEID :
         printf("Rule id is required.\n");
         break;
      case BSA_RC_OBJECT_NAME_TOO_LONG :
         printf("The object name is too long.\n");
         break;
      case BSA_RC_OBJECT_NOT_EMPTY :
         printf("The object is not empty.\n");
         break;
      case BSA_RC_OBJECT_NOT_FOUND :
         printf("There is no copy of the requested object.\n");
         break;
      case BSA_RC_OBJINFO_TOO_LONG :
         printf("The length of the object dependent\n");
         printf("information is too long.\n");
         break;
      case BSA_RC_OBJNAME_TOO_LONG :
         printf("The length of the object name is too long.\n");
         break;
      case BSA_RC_OPERATION_NOT_AUTHORIZED :
         printf("This session is not authorized to use this API\n");
         printf("call.\n");
         break;
      case BSA_RC_OLDTOKEN_REQD :
         printf("A value must be entered for the old token.\n");
         break;
      case BSA_RC_TOKEN_EXPIRED :
         printf("Security token has expired and must be\n");
         printf("updated.\n");
         break;
      case BSA_RC_TXN_ABORTED :
         printf("The transaction was aborted.\n");
         break;
      case BSA_RC_UNMATCHED_QUOTE :
         printf("An unmatched quote is in the value field of an\n");
         printf("environment variable.\n");
         break;
      case BSA_RC_USER_OWNS_OBJECTS :
         printf("A user cannot be deleted while it owns\n");
         printf("objects.\n");
         break;
      case ADSM_RC_ERROR :
         printf("ADSM error.  See the error log file.\n");
         break;
      case ADSM_RC_INVALID_NODE :
         printf("The BSAObjectOwner does not match the value\n");
         printf("on BSAInit.\n");
         break;
      case ADSM_RC_INVALID_COPYTYPE :
         printf("The value specified for copyType is invalid.\n");
         break;
      case ADSM_RC_INVALID_OBJTYPE :
         printf("The value specified for objectType is invalid.\n");
         break;
      case ADSM_RC_INVALID_STATUS :
         printf("The value specified for objectStatus is invalid.\n");
         break;
      case ADSM_RC_INVALID_ST_VER :
         printf("The value specified for the structure version is invalid.\n");
         break;
      case ADSM_RC_OWNER_TOO_LONG :
         printf("The value specified for the BSAObjectOwner or appObjectOwner is too long.\n");
         break;
      case ADSM_RC_PSWD_TOO_LONG :
         printf("The value specified for the securityToken is too long.\n");
         break;
      default :
         printf("There is no explanation for this error code.\n");
         break;
      } /* switch */

   return;
} /* rcXApiOut() */


