/***********************************************************************
* Name:
*        xapiutil.c
*
* Function:
*        Utility functions for the sample X/Open API code
*
* Public Routines:
*    format_number      : Format numbers with commas so they can be
*                         read
*    initialize_objDesc : allocates and initializes an object     
*                         descriptor
*
* Description:
*        The file contains utility functions for the X/Open sample API 
*        code that do not fit well within other source modules.
***********************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "xapiutil.h"       /* Handy utility routines.                 */
#include "xapint64.h"       /* 64 bit routines.                        */
#include "xapidef.h"        /* XOpen Sample API type definitions       */
#include "xapiour.h"        /* our API functions                       */

#define SEPARATOR 0xff      /* use x0ff as a separator of the          */
                            /* information stored in the object info   */
                            /* field                                   */

/*----------------------------------------------------------------------+
| Name:    format_number()
|
| Action:  Take a ascii string and format it with commas for readability.
|          Converts 123456789 to 123,456,789
|
| Input:   *s_in  - Pointer to input character string.
|
| Output:  *s_out - Pointer to output character buffer.
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   This routine handles formatted numbers up to 49 characters in
|          length, thus buffers of 50 bytes need be passed for the routine
|          to work.
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void format_number(s_in, s_out)
   char *s_in;
   char *s_out;
#else
void format_number(char *s_in, char *s_out)
#endif

{
char  *save;
char  temp[50];           /* Should be long enough for most numbers.      */
int   i;
char  separator = ',';    /* This could be a period in other countries.   */

if (!*s_in)
  {*s_out = 0x00; return;}

save = s_in;           /* Remember where it started.                */
s_in += strlen(s_in);  /* Point to end of string.                   */

i = 48;
temp[49] = 0;
while (s_in != save)
  {
  temp[i--] = (*s_in--);
  if (i%4==0) temp[i--] = separator;
  }
temp[i] = (*s_in);
strcpy(s_out,temp+i);
}

/*----------------------------------------------------------------------+
| Name:    create_objDesc() 
|
| Action:  Allocates memory for an object descriptor and initializes
|          the structure with the version number. 
|
| Input:   None 
|
| Output:  None
|
| Returns: *objectDescriptorPtr - returns a pointer to an initialized
|                                 object descriptor or NULL if there
|                                 is not enough memory 
|
| Side
| Effects: None
|
| Notes:   None
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
ObjectDescriptor* create_objDesc()
#else
ObjectDescriptor* create_objDesc()
#endif

{

ObjectDescriptor *objectDescriptorPtr;

objectDescriptorPtr = (ObjectDescriptor*)malloc( 
			       sizeof( ObjectDescriptor));
if (objectDescriptorPtr == NULL)
   {
   /* no memory available */
   return NULL;
   }

/* The objectDescriptorPtr -> version parameter is assumed to be       */
/* an input parameter. The current version number is defined in        */
/* custom.h .                                                          */
objectDescriptorPtr -> version = ObjectDescriptorVersion;
strcpy(objectDescriptorPtr -> Owner.bsaObjectOwner, "");
strcpy(objectDescriptorPtr -> Owner.appObjectOwner, "");
strcpy(objectDescriptorPtr -> objName.objectSpaceName, "");
strcpy(objectDescriptorPtr -> objName.pathName, "");
objectDescriptorPtr -> copyType = BSACopyType_BACKUP;
objectDescriptorPtr -> size = Make64( 0 );
strcpy(objectDescriptorPtr -> resourceType, "");
objectDescriptorPtr -> objectType = BSAObjectType_FILE;
objectDescriptorPtr -> encodingList = NULL;
strcpy(objectDescriptorPtr -> desc, "");
strcpy(objectDescriptorPtr -> objectInfo, "");

return objectDescriptorPtr;
}

/*----------------------------------------------------------------------+
| Name:    dispose_objDesc()
|
| Action:  Removes an object descriptor form the system. 
|
| Input:   *objectDescriptorPtr - pointer to the object descriptor
|                                 to be removed
|
| Output:
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void dispose_objDesc(objectDescriptorPtr)
   ObjectDescriptor *objectDescriptorPtr
#else
void dispose_objDesc(ObjectDescriptor *objectDescriptorPtr)
#endif

{

free( objectDescriptorPtr );
objectDescriptorPtr = NULL;
return;

}

/*----------------------------------------------------------------------+
| Name:    dispose_queryDesc()
|
| Action:  Removes an query descriptor form the system.
|
| Input:   *queryDescriptorPtr - pointer to the query descriptor
|                                to be removed
|
| Output:
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void dispose_queryDesc(queryDescriptorPtr)
   QueryDescriptor *queryDescriptorPtr
#else
void dispose_queryDesc(QueryDescriptor *queryDescriptorPtr)
#endif

{

free( queryDescriptorPtr );
queryDescriptorPtr = NULL;
return;

} /* dispose_queryDesc() */

/*----------------------------------------------------------------------+
| Name:    create_queryDesc()
|
| Action:  Allocates memory for a query descriptor and initializes
|          the structure.
|
| Input:
|
| Output:  None
|
| Returns: *queryDescriptorPtr - returns a pointer to an initialized
|                                query descriptor or NULL if there
|                                is not enough memory
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
QueryDescriptor* create_queryDesc()
#else
QueryDescriptor* create_queryDesc()
#endif

{

QueryDescriptor *queryDescriptorPtr;

queryDescriptorPtr = (QueryDescriptor*)malloc( 
				sizeof( QueryDescriptor));
if (queryDescriptorPtr == NULL)
   {
   /* no memory available */
   return NULL;
   }

strcpy(queryDescriptorPtr -> owner.bsaObjectOwner, "");
strcpy(queryDescriptorPtr -> owner.appObjectOwner, "");
strcpy(queryDescriptorPtr -> objName.objectSpaceName, "");
strcpy(queryDescriptorPtr -> objName.pathName, "");
queryDescriptorPtr -> copyType = BSACopyType_BACKUP;
strcpy(queryDescriptorPtr -> lGName, "");
strcpy(queryDescriptorPtr -> cGName, "");
strcpy(queryDescriptorPtr -> resourceType, "");
queryDescriptorPtr -> objectType = BSAObjectType_FILE;
queryDescriptorPtr -> status = BSAObjectStatus_ACTIVE; 
strcpy(queryDescriptorPtr -> desc, "*");

return queryDescriptorPtr;
} /* create_queryDesc() */

/*----------------------------------------------------------------------+
| Name:    create_dataBlock()
|
| Action:  Allocates memory for a data block descriptor and a data
|          buffer of the specified size. 
|
| Input:   size          - size of the buffer to create
|
| Output:  None
|
| Returns: *dataBlockPtr - returns a pointer to an initialized
|                          data block descriptor or NULL if there
|                          is not enough memory
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
DataBlock* create_dataBlock(size)
   BSA_UInt32 size;
#else
DataBlock* create_dataBlock(BSA_UInt32 size)
#endif

{

DataBlock *dataBlockPtr;
char      *bufferPtr;

dataBlockPtr = (DataBlock*)malloc( sizeof( DataBlock));
if (dataBlockPtr == NULL)
   {
   /* no memory available */
   return NULL;
   }
else
   {
   bufferPtr = create_buffer( size);
   if (bufferPtr == NULL)
      {
      /* no memory available */
      free(dataBlockPtr);
      dataBlockPtr = NULL;
      return NULL;
      }
   else
      {
      dataBlockPtr->bufferLen = size;
      dataBlockPtr->numBytes  = size;
      dataBlockPtr->bufferPtr = bufferPtr;
      }
   }

return dataBlockPtr;
}

/*----------------------------------------------------------------------+
| Name:    create_buffer()
|
| Action:  Allocates memory for a send or receive buffer. 
|
| Input:   size of the buffer to create
|
| Output:  None
|
| Returns: *bufferPtr - returns a pointer to a buffer or
|                       NULL if there is not enough memory
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
char* create_buffer(size)
   BSA_UInt32 size;
#else
char* create_buffer(BSA_UInt32 size)
#endif

{

char* bufferPtr;

bufferPtr = malloc( size );

return bufferPtr;
} /*  create_buffer() */

/*----------------------------------------------------------------------+
| Name:    dispose_buffer()
|
| Action:  Frees the buffer
|
| Input:   *bufferPtr - pointer to the buffer
|
| Output:
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void dispose_buffer(bufferPtr)
char *bufferPtr
#else
void dispose_buffer(char *bufferPtr)
#endif
  
{
free( bufferPtr );
return;
} /* dispose_buffer() */
 
/*----------------------------------------------------------------------+
| Name:    dispose_dataBlock()
|
| Action:  Memory for the data block descriptor and the data
|          buffer is given back to the system. 
|
| Input:   *dataBlockPtr - pointer to a data block descriptor 
|
| Output:  None
|
| Returns:
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void dispose_dataBlock()
   DataBlock *dataBlockPtr;
#else
void dispose_dataBlock(DataBlock *dataBlockPtr)
#endif

{

free(dataBlockPtr->bufferPtr);
dataBlockPtr->bufferPtr = NULL;
free(dataBlockPtr);
dataBlockPtr = NULL;

return;
}

/*----------------------------------------------------------------------+
| Name:    get_time() 
|
| Action:  Converts a string like yyyy/mm/dd hh:mm:ss to the tm structure 
|
| Input:   *timeString          - string containing data and time 
|          default              - default value for year          
|
| Returns: tm                   - time structure 
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
struct tm get_time()
  char              *timeString;
  BSA_Int16         default;
#else /* use ANSI C required prototypes */
struct tm get_time(char *timeString, BSA_Int16 default_limit)
#endif

{

struct tm   time;
char *token;
char *delimiters;
BSA_Int16 timeStringLen;
BSA_Int16 length;

if (default_limit == ADSM_HIGHEST_BOUND)
{
   default_limit = default_limit - 1900;
   /* avoid an overflow because API adds 1900 */
} /* if */
timeStringLen = strlen(timeString);
time.tm_year  = default_limit;
time.tm_mon   = 0;
time.tm_mday  = 1;
time.tm_hour  = 0;
time.tm_min   = 0;
time.tm_sec   = 0;

if (timeStringLen > 0)
{
   /*-------------------------------------------------------------------+
   | read year form timeString 
   .-------------------------------------------------------------------*/
   token = strtok(timeString, "/");
   length = strlen(token);
   if ((length > 0) && (length < 5))
   {
      time.tm_year = atoi(token);
      /* year since 1900 */
      if (time.tm_year >= 1900)
      {
         time.tm_year = time.tm_year - 1900;
      } /* if */
   } /* if */
   /*-------------------------------------------------------------------+
   | read month form timeString
   .-------------------------------------------------------------------*/
   token = strtok(NULL, "/");
   length = strlen(token);
   if ((length > 0) && (length < 3))
   {
      time.tm_mon = atoi(token);
      time.tm_mon = time.tm_mon - 1; /* 0 = January in ANSII C specs */
   } /* if */
   /*-------------------------------------------------------------------+
   | read day form timeString
   .-------------------------------------------------------------------*/
   token = strtok(NULL, " ");
   length = strlen(token);
   if ((length > 0) && (length < 3))
   {
      time.tm_mday = atoi(token);
   } /* if */
   /*-------------------------------------------------------------------+
   | read hour form timeString
   .-------------------------------------------------------------------*/
   token = strtok(NULL, ":");
   length = strlen(token);
   if ((length > 0) && (length < 3))
   {
      time.tm_hour = atoi(token);
   } /* if */
   /*-------------------------------------------------------------------+
   | read minute form timeString
   .-------------------------------------------------------------------*/
   token = strtok(NULL, ":");
   length = strlen(token);
   if ((length > 0) && (length < 3))
   {
      time.tm_min = atoi(token);
   } /* if */
   /*-------------------------------------------------------------------+
   | read second form timeString
   .-------------------------------------------------------------------*/
   token = strtok(NULL, " ");
   length = strlen(token);
   if ((length > 0) && (length < 3))
   {
      time.tm_sec = atoi(token);
   } /* if */
} /* if */

return time;
} /* get_time() */

/*----------------------------------------------------------------------+
| Name:    max16()
|
| Action:  calculates the maximum of the passed values and retruns it 
|
| Input:   x                    - first value
|          y                    - second value 
|
| Returns: BSA_Int16            - maximum of the input values 
|
| Side
| Effects: None
|
| Notes:
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 max16()
  BSA_Int16         x;
  BSA_Int16         y;
#else /* use ANSI C required prototypes */
BSA_Int16 max16(BSA_Int16 x, BSA_Int16 y)
#endif

{

BSA_Int16 result;

if (x > y)
{
   result = x;
}
else
{
   result = y;
} /* if */

return result;
} /* max16() */

/*----------------------------------------------------------------------+
| Name:    cpy_seed_to_buffer() 
|
| Action:  Fills the buffer by duplicating the seed string. 
|
| Input:   bufferPtr  - pointer to an allocated buffer
|          seedString - String to fill the buffer
|          length     - length of the buffer 
|
| Output:  *bufferPtr - buffer with duplicated seed string 
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   We use a simplistic brute force mechanism to stream data
|          to the server. The seed string is duplicated as needed
|          in sends (and buffers) until the requested object size is
|          reached.  For ease of demonstration, we just duplicate the
|          seed as many times as will fit in a buffer of size given in
|          the preferences.  
|          Whenever a seed string is entered on object sends
|          the send buffer size will be adjusted so that
|          multiple copies of the seed string fit exactly into the
|          buffer.
|          We then use this buffer over and over (or just part of it).
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void cpy_seed_to_buffer(bufferPtr, seedString, length)
   char       *bufferPtr;
   char       *seedString;
   BSA_UInt32 length; 
#else
void cpy_seed_to_buffer(char       *bufferPtr,
                        char       *seedString,
                        BSA_UInt32 length)
#endif

{

BSA_Int16 SeedSize;
char      *Cursor;
char      *EndOfBuff;
BSA_Int16 i;

SeedSize = strlen(seedString);

Cursor = bufferPtr;         /* Position within buffer. */
EndOfBuff = bufferPtr + length;
while (EndOfBuff >= (Cursor + SeedSize))
/* Duplicate seed in buff. */
{
   memcpy(Cursor, seedString, SeedSize);
   Cursor += SeedSize;
} /* while */
/* usually this loop will never be used because the copies of the */
/* seed string fit exactly into the buffer                        */
/* see the note above                                             */
if (EndOfBuff != Cursor)
{
   memcpy(Cursor, seedString, (EndOfBuff-Cursor));
} /* if */

} /* cpy_seed_to_buffer() */

/*----------------------------------------------------------------------+
| Name:    build_object_info 
|
| Action:  Stores eyecatcher, object size, copyType, seed string,
|          and object info in the object info field for verification
|          after object has been received. 
|
| Input:   *objectDescriptorPtr - pointer to the object descriptor
|          *SeedString          - pointer to the seed string 
|
| Output:  *objectDescriptorPtr - pointer to object descriptor with
|                                 modified info field
|
| Returns: BSA_RC_OK            - Successful
|          RC_NO_MEMORY         - there is not enough memory
|                                 or the control string in the object
|                                 information field is too long 
|
| Side
| Effects: None
|
| Notes:   The information stored in the object info field has the
|          following format:
|          eyecatcherSEPARATORSizeStringSEPARATOR
|          CopyTypeIndicatorSeedStringSEPARATORobjInfo
|          Between the CopyTypeIndicator and the SeedString is no
|          separator because the CopyTypeIndicator has a fixed
|          length. This string is terminated by 0x00.
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 build_object_info(objectDescriptorPtr, SeedString)
   ObjectDescriptor *objectDescriptorPtr
   char             *SeedString
#else
BSA_Int16 build_object_info(ObjectDescriptor *objectDescriptorPtr,
                            char             *SeedString)
#endif

{

BSA_UInt16 length;
BSA_UInt16 cursor;
char       newinfo[ADSM_MAX_OBJINFO];
BSA_Int16  rc;
char       *eyecatcher;
char       SizeString[50];
char       CopyTypeIndicator[2];
char       separator[2];

separator[0] = SEPARATOR;
separator[1] = '\0';
memset(CopyTypeIndicator, '\0', 2);
switch(objectDescriptorPtr->copyType)
{
   case(BSACopyType_ANY):
      CopyTypeIndicator[0] = 0x01;
      break;
   case(BSACopyType_ARCHIVE):
      CopyTypeIndicator[0] = 0x02;
      break;
   case(BSACopyType_BACKUP):
      CopyTypeIndicator[0] = 0x03;
      break;
   default:
      CopyTypeIndicator[0] = 0x03;
      printf("Failure in coding the build_object_info function.\n");
      printf("Assuming copy type: BSACopyType_BACKUP\n");
} /* switch */
I64toCh(objectDescriptorPtr->size, SizeString, 10);
eyecatcher = (void *)get_eyecatcher_copy();
if (eyecatcher == NULL)
{
   /* no memory available */
   rc = RC_NO_MEMORY;
}
else
{ 
   length = 0;
   /* length of eye catcher        */
   length = length + strlen(eyecatcher) + 1;
   /* length of object size string */
   length = length + strlen(SizeString) + 1;
   /* length of CopyTypeIndicator  */
   length = length + 1;
   /* length of the seed string */
   length = length + strlen(SeedString) + 1;
   /* length of object info     */
   length = length + strlen(objectDescriptorPtr->objectInfo) + 1;
   if (length > ADSM_MAX_OBJINFO)
   {
      printf("The control string in the object info field used");
      printf(" for data validation\n");
      printf("is too long. Use a shorter object information.\n");
      rc = RC_NO_MEMORY;
   }
   else
   {
      strcpy(newinfo, eyecatcher);
      strcat(newinfo, separator);
      strcat(newinfo, SizeString);
      strcat(newinfo, separator);
      strcat(newinfo, CopyTypeIndicator);
      strcat(newinfo, SeedString);
      strcat(newinfo, separator);
      strcat(newinfo, objectDescriptorPtr->objectInfo);
      strncpy(objectDescriptorPtr->objectInfo,
              newinfo,
              ADSM_MAX_OBJINFO - 1); 
                            /* terminating 0x00 will also be copied */
      free(eyecatcher);
      rc = BSA_RC_OK;
   } /* if */
} /* if */
   
return rc;

} /* build_object_info() */

/*----------------------------------------------------------------------+
| Name:    restore_object_info
|
| Action:  Checks the object info field for the eye catcher string
|          If eye catcher has been found the object size and copy
|          type will be compared with the current values in the
|          object descriptor and the original object info will be
|          restored.
|
| Input:   *objectDescriptorPtr - pointer to the object descriptor
|
| Output:  *objectDescriptorPtr - pointer to object descriptor with
|                                 original object info field if
|                                 eye catcher matches
|          **SeedStringPtr      - pointer to seed string if eye catcher has
|                                 been found or
|                                 empty string if eye catcher does
|                                 not match          
|                                 allocates at least one Byte of memory
|
| Returns: RC_OK                - object size and copy type is O.K.
|                                 or eye catcher string does not
|                                 match 
|          RC_CHECK_FAILURE     - validation failure
|          RC_NO_MEMORY         - there is not enough memory
|
| Side
| Effects: None
|
| Notes:   The information stored in the object info field has the
|          following format:
|          eyecatcherSEPARATORSizeStringSEPARATOR
|          CopyTypeIndicatorSeedStringSEPARATORobjInfo
|          Between the CopyTypeIndicator and the SeedString is no
|          separator because the CopyTypeIndicator has a fixed
|          length. This string is terminated by 0x00.
|
|          Do not forget to free the memory for the SeedString after
|          it has been used. This function allocates at least one Byte
|          of memory for the SeedString.
|          If the verbose flag is FALSE no printf commands will be
|          executed. This is used to restore the object info field
|          in the print_object_descriptor function (xapiqry module)
|          and in the send_object_and_data() function (xapisend module).
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 restore_object_info(objectDescriptorPtr, SeedStringPtr)
   ObjectDescriptor *objectDescriptorPtr
   char             **SeedStringPtr
#else
BSA_Int16 restore_object_info(ObjectDescriptor *objectDescriptorPtr,
                              char             **SeedStringPtr)
#endif

{

BSA_Int16   rc;
char        tmpInfo[ADSM_MAX_OBJINFO];
char        eyecatcher[ADSM_MAX_OBJINFO];
char        SizeString[ADSM_MAX_OBJINFO];
char        SeedString[ADSM_MAX_OBJINFO];
char        objInfo[ADSM_MAX_OBJINFO];
BSA_UInt64  objSize;
CopyType    copyType;
char        NumberString1[50];
char        NumberString2[50];
BSA_Boolean verbose;
char        separator[2];
char        *tokenPtr;

memset(tmpInfo, '\0', ADSM_MAX_OBJINFO);
memset(eyecatcher, '\0', ADSM_MAX_OBJINFO);
memset(SizeString, '\0', ADSM_MAX_OBJINFO);
memset(SeedString, '\0', ADSM_MAX_OBJINFO);
memset(objInfo, '\0', ADSM_MAX_OBJINFO);
separator[0] = SEPARATOR;
separator[1] = 0x00;
rc = RC_OK;
verbose = get_verbose();

strncpy(tmpInfo, objectDescriptorPtr->objectInfo, ADSM_MAX_OBJINFO - 1);
tokenPtr = strtok(tmpInfo, separator);
if (tokenPtr != NULL)
{
   strncpy(eyecatcher, tokenPtr, ADSM_MAX_OBJINFO - 1);
   tokenPtr = strtok(NULL, separator);
   if (tokenPtr != NULL)
   {
      strncpy(SizeString, tokenPtr, ADSM_MAX_OBJINFO - 1);
      tokenPtr = strtok(NULL, separator);
      if (tokenPtr != NULL)
      {
         strncpy(SeedString, tokenPtr, ADSM_MAX_OBJINFO - 1);
         switch(SeedString[0])
         {
            case(0x01):
               copyType = BSACopyType_ANY;
               break;
            case(0x02):
               copyType = BSACopyType_ARCHIVE;
               break;
            case(0x03):
               copyType = BSACopyType_BACKUP;
               break;
            default:
               if (verbose)
               {
                  copyType = BSACopyType_BACKUP;
                  printf("The control string in the object information field");
                  printf(" contains an invalid value.\n");
                  printf("Assuming: BSACopyType_BACKUP\n");
               } /* if */
         } /* switch */
         memmove(SeedString, &SeedString[1], strlen(SeedString));
         tokenPtr = strtok(NULL, separator);
         if (tokenPtr != NULL)
         {
            strncpy(objInfo, tokenPtr, ADSM_MAX_OBJINFO - 1);
         } /* if */
      } /* if */
   } /* if */
} /* if */
if (is_equal_eyecatcher(eyecatcher) == TRUE)
{
   if (copyType != objectDescriptorPtr->copyType)
   {
      rc = RC_CHECK_FAILURE;
      if (verbose)
      {
         printf("Different object types of original and ");
         printf("received object: %s %s\n",
                objectDescriptorPtr->objName.objectSpaceName,
                objectDescriptorPtr->objName.pathName);
         printf("The copy Type of the received object descriptor is ");
         switch(objectDescriptorPtr->copyType)
         {
            case(BSACopyType_ANY):
               printf("'ANY'");
               break;
            case(BSACopyType_ARCHIVE):
               printf("archive");
               break;
            case(BSACopyType_BACKUP):
               printf("backup");
               break;
            default:
               printf("an unkown value");
         } /* switch */
         printf(".\n");
         printf("The original copy type was ");
         switch(copyType)
         {
            case(BSACopyType_ANY):
               printf("'ANY'");
               break;
            case(BSACopyType_ARCHIVE):
               printf("archive");
               break;
            case(BSACopyType_BACKUP):
               printf("backup");
               break;
            default:
               printf("an unknown value");
         } /* switch */
         printf(".\n");
      } /* if */
   } /* if */
   objSize = ChtoI64(SizeString, 10);
   if (Cmp64(objSize, objectDescriptorPtr->size) != EQUAL)
   {
      rc = RC_CHECK_FAILURE;
      if (verbose)
      {
         printf("Different object sizes of original and ");
         printf("received object: %s %s\n",
                objectDescriptorPtr->objName.objectSpaceName,
                objectDescriptorPtr->objName.pathName);
         I64toCh(objectDescriptorPtr->size, NumberString1, 10);
         I64toCh(objSize, NumberString2, 10);
         printf("The object size in the received object descriptor is %s.\n",
                NumberString1);
         printf("The original object size was %s.\n",
                NumberString2);
      } /* if */
   } /* if */
   *SeedStringPtr = malloc(strlen(SeedString) + 1);
   strcpy(*SeedStringPtr, SeedString);
   /*
   if (verbose)
   {
      printf("The seed string in the object info field is: =>%s<=\n",
             *SeedStringPtr);
   }
   */
   /*
   if (verbose)
   {
      printf("The object info is: %s\n",
             objInfo);
   }
   */
   strcpy(objectDescriptorPtr->objectInfo, objInfo);
} /* if */

return rc;
} /* restore_object_info() */

/*----------------------------------------------------------------------+
| Name:    check_Data()
|
| Action:  Checks the data of the buffer against the seed string
|
| Input:   *dataBlockPtr        - pointer to the data block
|          *seedString          - seed string
|          *offsetPtr           - pointer to an offset used for
|                                 data validation
| 
| Output:  *offsetPtr           - pointer to an offset used for
|                                 data validation
|
| Returns: RC_OK                - data in the buffer is O.K.
|          RC_CHECK_FAILURE     - validation failure
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
BSA_Int16 check_Data        (dataBlockPtr,
                             seedString,
                             offsetPtr)
   DataBlock        *dataBlockPtr;
   char             *seedString;
   BSA_UInt16       *offsetPtr;
#else
BSA_Int16 check_Data(DataBlock        *dataBlockPtr,
                     char             *seedString,
                     BSA_UInt16       *offsetPtr)
 #endif
{

BSA_Int16       rc;
BSA_UInt16      i;
BSA_UInt16      seedLen;
DataBlock       *reference_dataBlockPtr;

i = 0;
rc = RC_OK;
seedLen = strlen(seedString);
reference_dataBlockPtr = create_dataBlock(
                          (dataBlockPtr->bufferLen) + *offsetPtr);
cpy_seed_to_buffer(reference_dataBlockPtr->bufferPtr,
                   seedString,
                   (dataBlockPtr->bufferLen) + *offsetPtr);

while ((rc == RC_OK) &&
       (i < dataBlockPtr->numBytes))
{
   if (reference_dataBlockPtr->bufferPtr[i+(*offsetPtr)] !=
                                   dataBlockPtr->bufferPtr[i])
   {
      rc = RC_CHECK_FAILURE;
   }
   else
   {
      i++;
   } /* if */
} /* while */
*offsetPtr = (*offsetPtr + 
             (BSA_UInt16)(((dataBlockPtr->bufferLen)%seedLen))
             %seedLen);
dispose_dataBlock(reference_dataBlockPtr);
return rc;
} /* check_Data() */

/*----------------------------------------------------------------------+
| Name:    create_objOwner() 
|
| Action:  Allocates memory for an object owner and initializes
|          the structure with 0x00 chracters. 
|
| Input:   None
|
| Output:  None
|
| Returns: *objectOwnerPtr - returns a pointer to an initialized
|                            object owner or NULL if there
|                            is not enough memory 
|
| Side
| Effects: None
|
| Notes:   None
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
ObjectOwner* create_objOwner()
#else
ObjectOwner* create_objOwner()
#endif

{

ObjectOwner *objectOwnerPtr;

objectOwnerPtr = (ObjectOwner*)malloc( sizeof( ObjectOwner));
if (objectOwnerPtr != NULL)
{
   memset(objectOwnerPtr->bsaObjectOwner, '\0', BSA_MAX_BSAOBJECT_OWNER);
   memset(objectOwnerPtr->appObjectOwner, '\0', BSA_MAX_APPOBJECT_OWNER);
}
else
{
   /* no memory available */
} /* if */

return objectOwnerPtr;
} /* create_objOwner() */

/*----------------------------------------------------------------------+
| Name:    dispose_objOwner()
|
| Action:  Removes an object owner form the system.
|
| Input:   *objectOwnerPtr - pointer to the object owner
|                            to be removed
|
| Output:  None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
|
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void dispose_objOwner(objOwnerPtr)
   ObjectOwner *objectOwnerPtr
#else
void dispose_objOwner(ObjectOwner *objectOwnerPtr)
#endif

{

free( objectOwnerPtr );
objectOwnerPtr = NULL;
return;

} /* dispose_objOwner() */

/*----------------------------------------------------------------------+
| Name:    print_object_descriptor() 
|
| Action:  Prints one object descriptor. 
|
| Input:   *objectDescriptorPtr      - object descriptor 
|
| Output:  None
|
| Returns: None
|
| Side
| Effects: None
|
| Notes:   None
+----------------------------------------------------------------------*/
#ifdef _NO_PROTO_
void print_object_descriptor(objectDescriptorPtr)
   ObjectDescriptor *objectDescriptorPtr;
#else
void print_object_descriptor(ObjectDescriptor *objectDescriptorPtr)
#endif
{

char        NumberString[50];
char        TimeString[51];
char        FormatString[] = "%B %d %Y %H:%M:%S";
BSA_Boolean tmpVerbose;
char        *SeedString = NULL;

if (get_qry_detail() == FALSE)
{
   printf("   Object Space Name: %s\n", 
          objectDescriptorPtr->objName.objectSpaceName); 
   printf("           Path Name: %s\n",
          objectDescriptorPtr->objName.pathName);
}
else
{
   printf("                       Object Space Name: %s\n", 
          objectDescriptorPtr->objName.objectSpaceName); 
   printf("                               Path Name: %s\n",
          objectDescriptorPtr->objName.pathName);
   printf("                             Object Type: ");
   switch (objectDescriptorPtr->objectType)
   {
      case BSAObjectType_ANY:
         printf("ANY");
         break;
      case BSAObjectType_FILE:
         printf("FILE");
         break;
      case BSAObjectType_DIRECTORY:
         printf("DIRECTORY");
         break;
      case BSAObjectType_DATABASE:
         printf("DATABASE");
         break;
      default:
         printf("Invalid value!");
         break;
   } /* switch */
   printf("\n");
   printf("                           Resource Type: %s\n",
          objectDescriptorPtr->resourceType);
   strftime(TimeString, 50, FormatString,
            &objectDescriptorPtr->createTime);
   printf("                             Create time: %s\n",
          TimeString);
   printf("                   BSA Object Owner Name: %s\n",
          objectDescriptorPtr->Owner.bsaObjectOwner);
   printf("           Application Object Owner Name: %s\n",
          objectDescriptorPtr->Owner.appObjectOwner);
   printf("                                  Status: ");
   switch (objectDescriptorPtr->status)
   {
      case BSAObjectStatus_ANY:
         printf("ANY");
         break;
      case BSAObjectStatus_ACTIVE:
         printf("ACTIVE");
         break;
      case BSAObjectStatus_INACTIVE:
         printf("INACTIVE");
         break;
      default:
         printf("Invalid value!");
         break;
   } /* switch */
   printf("\n");
   printf("         Associated Lifecycle Group name: %s\n",
          objectDescriptorPtr->lGName);
   /* do not print this for now
   printf("   Copy Group within the lifecycle group: %s\n",
          objectDescriptorPtr->cGName);
   */
   printf("                       Descriptive Label: %s\n",
          objectDescriptorPtr->desc);
   /* suppress any printf commands in restore_object_info */
   tmpVerbose = get_verbose();
   set_verbose(FALSE);
   restore_object_info(objectDescriptorPtr, &SeedString);
   free(SeedString);
   set_verbose(tmpVerbose);
   printf("                      Object Information: %s\n",
          objectDescriptorPtr->objectInfo);
   printf("                               Copy Type: ");
   switch (objectDescriptorPtr->copyType)
   {
      case BSACopyType_ANY:
         printf("ANY");
         break;
      case BSACopyType_ARCHIVE:
         printf("ARCHIVE");
         break;
      case BSACopyType_BACKUP:
         printf("BACKUP");
         break;
      default:
         printf("Invalid value!");
         break;
   } /* switch */
   printf("\n");
   I64toCh(objectDescriptorPtr->size, NumberString, 10);
   printf("                             Object Size: %s\n",
          NumberString);
   I64toCh(objectDescriptorPtr->restoreOrder, NumberString, 10);
   printf("                           Restore Order: %s\n",
          NumberString);
   I64toCh(objectDescriptorPtr->copyId, NumberString, 10);
   printf("                                 Copy ID: %s\n",
          NumberString);
} /* if */

return;
} /* print_object_descriptor() */

